<?php

declare(strict_types=1);

namespace Mefi\Inkasso\Administration\Controller;

use DateTime;
use Shopware\Core\Framework\Context;
use Shopware\Core\Framework\Routing\Annotation\RouteScope;
use Shopware\Core\Framework\Routing\Exception\InvalidRequestParameterException;
use Shopware\Core\Framework\Validation\DataBag\RequestDataBag;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\Routing\Annotation\Route;
use Mefi\Inkasso\Components\Claim\ClaimServiceInterface;
use Mefi\Inkasso\Components\Claim\Hydrator\NewClaimFormHydratorInterface;

/**
 * @RouteScope(scopes={"api"})
 */
class MediafinanzClaimController extends AbstractController
{
    /** @var ClaimServiceInterface */
    private $claimService;

    /** @var NewClaimFormHydratorInterface */
    private $newClaimFormHydrator;

    public function __construct(
        ClaimServiceInterface $claimService,
        NewClaimFormHydratorInterface $newClaimFormHydrator
    ) {
        $this->claimService         = $claimService;
        $this->newClaimFormHydrator = $newClaimFormHydrator;
    }

    /**
     * @Route("/api/_action/mediafinanz-claim/new-claim", name="api.action.mefi.inkasso.claim.new", methods={"POST"})
     */
    public function newClaim(RequestDataBag $request, Context $context): JsonResponse
    {
        $newClaimForm = $this->newClaimFormHydrator->hydrate($request);
        $claimResult  = $this->claimService->createNewClaim($newClaimForm, $context);

        return new JsonResponse($claimResult->jsonSerialize());
    }

    /**
     * @Route("/api/_action/mediafinanz-claim/close-claim", name="api.action.mefi.inkasso.claim.close", methods={"POST"})
     */
    public function closeClaim(RequestDataBag $request): JsonResponse
    {
        $fileNumber = $request->get('fileNumber', null);

        if (empty($fileNumber)) {
            throw new InvalidRequestParameterException('fileNumber');
        }

        $result = $this->claimService->closeClaim($fileNumber);

        return new JsonResponse($result->jsonSerialize());
    }

    /**
     * @Route("/api/_action/mediafinanz-claim/book-direct-payment", name="api.action.mefi.inkasso.claim.book-direct-payment", methods={"POST"})
     */
    public function bookDirectPayment(RequestDataBag $request, Context $context): JsonResponse
    {
        $fileNumber    = $request->get('fileNumber');
        $dateOfPayment = $request->get('dateOfPayment', (new DateTime())->format('Y-m-d'));
        $paidAmount    = $request->get('paidAmount');

        if (empty($paidAmount)) {
            throw new InvalidRequestParameterException('paidAmount');
        }

        if (empty($fileNumber)) {
            throw new InvalidRequestParameterException('fileNumber');
        }

        $result = $this->claimService->bookDirectPayment($fileNumber, $dateOfPayment, $paidAmount, $context);

        return new JsonResponse($result->jsonSerialize());
    }

    /**
     * @Route("/api/_action/mediafinanz-claim/get-claim-status", name="api.action.mefi.inkasso.claim.get-status", methods={"POST"})
     */
    public function getClaimStatus(RequestDataBag $request): JsonResponse
    {
        $fileNumber = $request->get('fileNumber');

        if (empty($fileNumber)) {
            throw new InvalidRequestParameterException('fileNumber');
        }

        $result = $this->claimService->getClaimStatus($fileNumber);

        return new JsonResponse($result->jsonSerialize());
    }
}
