<?php

declare(strict_types=1);

namespace Mefi\Inkasso\Administration\Controller;

use Shopware\Core\Checkout\Customer\Aggregate\CustomerAddress\CustomerAddressEntity;
use Shopware\Core\Checkout\Customer\CustomerEntity;
use Shopware\Core\Framework\Context;
use Shopware\Core\Framework\DataAbstractionLayer\EntityRepositoryInterface;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Criteria;
use Shopware\Core\Framework\Routing\Annotation\RouteScope;
use Shopware\Core\Framework\Validation\DataBag\RequestDataBag;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\Routing\Annotation\Route;
use Symfony\Component\Validator\ConstraintViolationList;
use Mefi\Inkasso\Components\Solvency\SolvencyCachingServiceInterface;
use Mefi\Inkasso\Components\Solvency\SolvencyScoreAdapter;
use Mefi\Inkasso\Components\Solvency\Struct\CustomerInfoStruct;
use Mefi\Inkasso\Components\Solvency\Struct\ScoreResult;
use Mefi\Inkasso\Components\Solvency\Validation\ScoreValidator;
use Mefi\Inkasso\Components\Solvency\Validation\ScoreValidatorInterface;
use Mefi\Inkasso\Components\Solvency\Validation\Violation\AbstractScoreViolation;
use Mefi\Inkasso\Utils\AddressHelper;
use Mefi\Inkasso\Components\SoapAPI\Buergel\Request\Struct\ConCheckScoreRequestStruct;

/**
 * @RouteScope(scopes={"api"})
 */
class MediafinanzSolvencyHistoryController extends AbstractController
{
    /** @var EntityRepositoryInterface */
    private $customerRepository;

    /** @var EntityRepositoryInterface */
    private $customerAddressRepository;

    /** @var SolvencyScoreAdapter */
    private $solvencyScoreAdapter;

    /** @var SolvencyCachingServiceInterface */
    private $solvencyCachingService;

    /** @var ScoreValidatorInterface */
    private $scoreValidator;

    public function __construct(
        SolvencyScoreAdapter $solvencyScoreAdapter,
        SolvencyCachingServiceInterface $solvencyCachingService,
        ScoreValidatorInterface $scoreValidator,
        EntityRepositoryInterface $customerRepository,
        EntityRepositoryInterface $customerAddressRepository
    ) {
        $this->customerRepository        = $customerRepository;
        $this->customerAddressRepository = $customerAddressRepository;
        $this->solvencyScoreAdapter      = $solvencyScoreAdapter;
        $this->solvencyCachingService    = $solvencyCachingService;
        $this->scoreValidator            = $scoreValidator;
    }


    /**
     * @Route("/api/_action/mediafinanz-solvency-history/perform-new-solvency-check", name="api.action.mefi.inkasso.solvency.history.perform.new.solvency.check", methods={"POST"})
     */
    public function performNewSolvencyCheck(RequestDataBag $requestData, Context $context): JsonResponse
    {

        $customerId = $requestData->get('customerId', null);
        $addressId  = $requestData->get('customerAddressId', null);
        $product    = $requestData->get('product', null);
        $amount     = $requestData->get('amount', 0.0);
        $justification = (int) $requestData->get( 'justification', 3);

        if (empty($addressId) || empty($customerId)) {
            return new JsonResponse(['success' => false]);
        }

        //Prepare all the data we need to perform a solvency check...
        /** @var null|CustomerAddressEntity $customerAddress */
        $customerAddress = $this->customerAddressRepository->search(new Criteria([$addressId]), $context)->first();


        /** @var null|CustomerEntity $customer */
        $customer = $this->customerRepository->search(new Criteria([$customerId]), $context)->first();

        if ($customer === null || $customerAddress === null) {
            return new JsonResponse(['success' => false]);
        }


        $mockedOrderAddress = AddressHelper::createOrderAddressByAddress($customerAddress);

        if (!$mockedOrderAddress) {
            return new JsonResponse(['success' => false]);
        }


        $customerInformation = new CustomerInfoStruct($mockedOrderAddress, $customer);


        //Fetch the solvency from the API
        $score = $this->solvencyScoreAdapter->fetchScore(
            $customerInformation,
            $product,
            $amount,
            $customer->getSalesChannelId(),
            $justification,
            $context,
            false
        );

        $error = $score->getErrors() !== null ? $score->getErrors()->first() : null;

        if ($error !== null) {

            return new JsonResponse(['success' => false, 'error' => [
                'code'       => $error->getCode(),
                'message'    => $error->getMessage(),
                'apiMessage' => $error->getMessage()
                ]
            ]);
        }

        /* Validate the new score
        $violations = $this->scoreValidator->validate(
            $score,
            $customerInformation,
            $amount,
            $customer->getSalesChannelId(),
            $context
        );

        if ($violations->count() > 0) {
            return new JsonResponse(['success' => false, 'error' => $this->getSolvencyDeclinedMessage(
                $violations,
                $score
            )]);
        }
        */

        return new JsonResponse(['success' => true, 'data' => ['score' => $score->jsonSerialize()]]);
    }

    /*private function getSolvencyDeclinedMessage(ConstraintViolationList $violationList, ScoreResult $scoreResult): array
    {

        $error = $scoreResult->getErrors() !== null ? $scoreResult->getErrors()->first() : null;

        if ($error !== null) {
            return [
                'code'       => $error->getCode(),
                'message'    => $error->getMessage(),
                'apiMessage' => $error->getMessage(),
            ];
        }

        if ($violationList->count() > 0) {
            $violation  = $violationList->get(0);
            $apiMessage = $violation->getMessage();

            if ($violation instanceof AbstractScoreViolation) {
                $apiMessage = $violation->getOriginalMessage();
            }

            return [
                'code'       => $violation->getCode(),
                'message'    => $violation->getMessage(),
                'apiMessage' => $apiMessage,
            ];
        }

        return [
            'code' => ScoreValidator::VIOLATION_API_ERROR_GENERIC,
        ];
    }*/
}
