<?php

declare(strict_types=1);

namespace Mefi\Inkasso\Components\Claim;

use DateTime;
use Mefi\Inkasso\Components\Solvency\ReferenceProviderInterface;
use Psr\Log\LoggerInterface;
use RuntimeException;
use Shopware\Core\Framework\Context;
use Shopware\Core\Framework\DataAbstractionLayer\EntityRepositoryInterface;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Criteria;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Filter\EqualsFilter;
use Shopware\Core\Framework\Struct\Collection;
use Symfony\Component\Config\Definition\Exception\Exception;
use Mefi\Inkasso\Components\Claim\Struct\BookDirectPaymentResult;
use Mefi\Inkasso\Components\Claim\Struct\ClaimResult;
use Mefi\Inkasso\Components\Claim\Struct\ClaimStatusResult;
use Mefi\Inkasso\Components\Claim\Struct\CloseClaimResult;
use Mefi\Inkasso\Components\Claim\Struct\NewClaimFormStruct;
use Mefi\Inkasso\Components\SoapAPI\Encashment\DataTypes\ClaimIdentifier;
use Mefi\Inkasso\Components\SoapAPI\Encashment\DataTypes\DirectPayment;
use Mefi\Inkasso\Components\SoapAPI\Encashment\Request\BookDirectPaymentRequest;
use Mefi\Inkasso\Components\SoapAPI\Encashment\Request\Builder\GetClaimStatusRequestBuilderInterface;
use Mefi\Inkasso\Components\SoapAPI\Encashment\Request\Builder\NewClaimRequestBuilderInterface;
use Mefi\Inkasso\Components\SoapAPI\Encashment\Request\ClaimStatusRequest;
use Mefi\Inkasso\Components\SoapAPI\Encashment\Request\CloseClaimRequest;
use Mefi\Inkasso\Components\SoapAPI\Encashment\Request\NewClaimRequest;
use Mefi\Inkasso\Components\SoapAPI\Encashment\Request\Struct\BookDirectPaymentRequestStruct;
use Mefi\Inkasso\Components\SoapAPI\Encashment\Request\Struct\CloseClaimRequestStruct;
use Mefi\Inkasso\Components\SoapAPI\Encashment\Response\ClaimStatusResponse;
use Mefi\Inkasso\Components\SoapAPI\Encashment\Response\NewClaimResponse;
use Mefi\Inkasso\Components\SoapAPI\MediafinanzSoapClientInterface;
use Mefi\Inkasso\Core\DataAbstractionLayer\Claim\ClaimEntity;

class ClaimService implements ClaimServiceInterface
{
    /** @var NewClaimRequestBuilderInterface */
    private $newClaimRequestBuilder;

    /** @var MediafinanzSoapClientInterface */
    private $client;

    /** @var EntityRepositoryInterface */
    private $claimRepository;

    /** @var EntityRepositoryInterface */
    private $claimDirectPaymentRepository;

    /** @var LoggerInterface */
    private $logger;

    /** @var GetClaimStatusRequestBuilderInterface */
    private $getClaimStatusRequestBuilder;

    /** @var ReferenceProviderInterface */
    private $referenceProvider;

    public function __construct(
        MediafinanzSoapClientInterface $client,
        NewClaimRequestBuilderInterface $newClaimRequestBuilder,
        GetClaimStatusRequestBuilderInterface $getClaimStatusRequestBuilder,
        EntityRepositoryInterface $claimRepository,
        EntityRepositoryInterface $claimDirectPaymentRepository,
        ReferenceProviderInterface $referenceProvider,
        LoggerInterface $logger
    ) {
        $this->client                       = $client;
        $this->newClaimRequestBuilder       = $newClaimRequestBuilder;
        $this->claimRepository              = $claimRepository;
        $this->claimDirectPaymentRepository = $claimDirectPaymentRepository;
        $this->getClaimStatusRequestBuilder = $getClaimStatusRequestBuilder;
        $this->referenceProvider            = $referenceProvider;
        $this->logger                       = $logger;
    }

    public function createNewClaim(NewClaimFormStruct $form, Context $context): ClaimResult
    {

        /*
         * Bemerkung (note) um Referenz-String ergänzen
         */
        $request     = new NewClaimRequest();
        $requestData = $this->newClaimRequestBuilder->getData($form, $context);

        $claim = $requestData->getClaim();
        $swReference = $this->getReference(). $claim->getNote();

        $claim->setNote($swReference);

        $requestData->setClaim($claim);
        $request->setRequestData($requestData);

        $response = new NewClaimResponse(
            $this->client->sendRequest($request)
        );

        if ($response->hasErrors()) {
            $errorMessage    = 'Unknown error';
            $errorCollection = $response->getErrors();

            if ($errorCollection !== null && $errorCollection->count() > 0) {
                $errors       = $errorCollection->jsonSerialize();
                $latestError  = $errors[0];
                $errorMessage = $latestError->getMessage();
            }


            $claimResult = new ClaimResult();
            $claimResult->setSuccess(false);
            $claimResult->setErrorMessage($errorMessage);

            $this->logger->error($errorMessage);

            return $claimResult;
        }

        $result = new ClaimResult();
        $result->setNewClaimResult($response->getNewClaimResult());

        try {
            $this->insertToDB($form, $result, $context);
            $result->setSuccess(true);
        } catch (Exception $e) {
            $result->setSuccess(false);
            $result->setErrorMessage($e->getMessage());
        }

        return $result;
    }

    public function closeClaim(string $fileNumber): CloseClaimResult
    {
        $claimIdentifier = new ClaimIdentifier($fileNumber);

        $claimRequestData = new CloseClaimRequestStruct();
        $claimRequestData->setClaimIdentifier($claimIdentifier);

        $request = new CloseClaimRequest($claimRequestData);

        $response = $this->client->sendRequest($request);

        return new CloseClaimResult($claimIdentifier, $response === true);
    }

    public function getClaim(string $fileNumber, Context $context): ?ClaimEntity
    {
        $criteria = new Criteria();
        $criteria->addFilter(
            new EqualsFilter('fileNumber', $fileNumber)
        );

        return $this->claimRepository->search($criteria, $context)->first();
    }

    public function bookDirectPayment(
        string $fileNumber,
        string $dateOfPayment,
        float $paidAmount,
        Context $context
    ): BookDirectPaymentResult {
        $claimIdentifier = new ClaimIdentifier($fileNumber);

        //Convert date (string) to the correct format, since it might differ when sent from the administration.
        $dateOfPaymentTimestamp = strtotime($dateOfPayment);

        if ($dateOfPaymentTimestamp === false) {
            throw new RuntimeException('The dateOfPayment property could not be converted to the format expected by the encashment API');
        }

        $dateOfPayment = (new DateTime())->setTimestamp($dateOfPaymentTimestamp)->format('Y-m-d');

        $directPayment = new DirectPayment($dateOfPayment, $paidAmount);

        $requestData = new BookDirectPaymentRequestStruct();
        $requestData->setDirectPayment($directPayment);
        $requestData->setClaimIdentifier($claimIdentifier);

        $request = new BookDirectPaymentRequest($requestData);

        $response = $this->client->sendRequest($request);

        if (true === $response)
        {
            try {
                $this->insertDirectPaymentToDB($directPayment, $claimIdentifier, $context);
            } catch (Exception $e) {
                // error logging ...
                $this->logger->error($e->getMessage());
            }
        }

        return new BookDirectPaymentResult($response === true);
    }

    public function getClaimStatus(string $fileNumber): ClaimStatusResult
    {
        $request = new ClaimStatusRequest($this->getClaimStatusRequestBuilder->getData($fileNumber));

        $response = $this->client->sendRequest($request);

        $claimStatusResponse = new ClaimStatusResponse($response);

        if ($claimStatusResponse->getErrors() !== null) {
            return new ClaimStatusResult(false, null, $claimStatusResponse->getHumanReadableErrorList());
        }

        return new ClaimStatusResult(true, $claimStatusResponse->getStatus());
    }

    private function insertToDB(NewClaimFormStruct $entityFormData, ClaimResult $result, Context $context): void
    {
        $newClaimResult = $result->getNewClaimResult();

        if (!$newClaimResult) {
            throw new Exception('Missing claim result');
        }

        $this->claimRepository->upsert([[
            'invoiceNumber' => $entityFormData->getInvoiceNumber(),
            'fileNumber'    => $newClaimResult->getFilenumber(),
            'customerId'    => $entityFormData->getCustomerId(),
            'orderId'       => $entityFormData->getOrderId(),
            'claimDetail'   => [
                'fileNumber'         => $newClaimResult->getFilenumber(),
                'timeOfTransmission' => date('Y-m-d H:i:s'),
                'amount'             => (string) $entityFormData->getClaimAmount(),
                'status'             => $entityFormData->getStatus(),
                'type'               => (string) $entityFormData->getClaimType(),
                'reason'             => $entityFormData->getClaimReason(),
                'cost'               => (string) $entityFormData->getClaimFee(),
                'dateOfLastReminder' => $entityFormData->getDateOfLastReminder(),
                'notice'             => $entityFormData->getNote(),
            ],
            'claimCustomer' => [
                'company'   => $entityFormData->getCompany(),
                'lastName'  => $entityFormData->getLastName(),
                'firstName' => $entityFormData->getFirstName(),
                'street'    => $entityFormData->getStreet(),
                'zipCode'   => $entityFormData->getZipCode(),
                'city'      => $entityFormData->getCity(),
                'countryId' => $entityFormData->getCountryId(),
                'phone'     => $entityFormData->getPhone(),
                'email'     => $entityFormData->getEmail(),
            ],
        ]], $context);
    }

    private function insertDirectPaymentToDB(DirectPayment $directPayment, ClaimIdentifier $claimIdentifier, Context $context): void
    {
        $this->logger->debug(__METHOD__);

        $claimEntity = $this->getClaim($claimIdentifier->getFileNumber(), $context);

        $this->logger->debug("[" . __LINE__ . "]\tClaimUUID: " . var_export($claimEntity->getId(), true) . "\n");

        $this->claimDirectPaymentRepository->upsert([[
            'claimId' => $claimEntity->getId(),
            'invoiceNumber' => $claimEntity->getInvoiceNumber(),
            'fileNumber' => $claimIdentifier->getFileNumber(),
            'paidAmount' => $directPayment->getPaidAmount(),
            'dateOfPayment' => $directPayment->getDateOfPayment(),
        ]],$context);

    }

    private function getReference(): ?string
    {
        $context = Context::createDefaultContext();

        return $this->referenceProvider->getReference($context);
    }
}
