<?php

declare(strict_types=1);

namespace Mefi\Inkasso\Components\Claim;

use Exception;
use Psr\Log\LoggerInterface;
use Shopware\Core\Framework\Context;
use Shopware\Core\Framework\DataAbstractionLayer\EntityRepositoryInterface;
use Mefi\Inkasso\Components\Claim\Struct\ClaimStatusChangesFetchResult;
use Mefi\Inkasso\Components\SoapAPI\Encashment\DataTypes\ClaimStatusChange;
use Mefi\Inkasso\Components\SoapAPI\Encashment\Request\ClaimStatusChangesRequest;
use Mefi\Inkasso\Components\SoapAPI\Encashment\Response\ClaimStatusChangesResponse;
use Mefi\Inkasso\Components\SoapAPI\Encashment\Service\CommitTransactionServiceInterface;
use Mefi\Inkasso\Components\SoapAPI\MediafinanzSoapClientInterface;

class ClaimStatusHistoryService implements ClaimStatusHistoryServiceInterface
{
    /** @var MediafinanzSoapClientInterface */
    private $client;

    /** @var EntityRepositoryInterface */
    private $claimStatusHistoryRepository;

    /** @var LoggerInterface */
    private $logger;

    /** @var CommitTransactionServiceInterface */
    private $commitTransactionService;

    /** @var ClaimServiceInterface */
    private $claimService;

    public function __construct(
        MediafinanzSoapClientInterface $encashmentClient,
        EntityRepositoryInterface $claimStatusHistoryRepository,
        CommitTransactionServiceInterface $commitTransactionService,
        ClaimServiceInterface $claimService,
        LoggerInterface $logger
    ) {
        $this->client                       = $encashmentClient;
        $this->claimStatusHistoryRepository = $claimStatusHistoryRepository;
        $this->logger                       = $logger;
        $this->commitTransactionService     = $commitTransactionService;
        $this->claimService                 = $claimService;
    }

    /**
     * {@inheritdoc}
     */
    public function fetchClaimStatusChanges(Context $context): ClaimStatusChangesFetchResult
    {
        $response = new ClaimStatusChangesResponse(
            $this->client->sendRequest(new ClaimStatusChangesRequest()
        ));

        if ($response->hasErrors() && $response->getErrors() !== null) {
            $this->logger->error('Could not fetch claim status changes from API!', [
                'errors' => $response->getErrors(),
            ]);

            return new ClaimStatusChangesFetchResult(0, $response->getHumanReadableErrorList(), false);
        }

        if ($response->getChanges() !== null && count($response->getChanges()) === 0) {
            return new ClaimStatusChangesFetchResult(0, null, true);
        }

        $statusChanges = $response->getChanges() ?? [];
        $transactionId = $response->getTransactionId();

        try {
            if (!empty($statusChanges)) {
                $this->insertClaimStatusChanges($statusChanges, $context);
            }

            if (!empty($transactionId)) {
                $this->commitTransactionService->commitTransaction($transactionId);
            }
        } catch (Exception $e) {
            $this->logger->warning('Claim status history: Could not insert claim status change to the database due to an error', [
                'transactionId' => $response->getTransactionId(),
                'exception'     => $e->getMessage(),
                'trace'         => $e->getTraceAsString(),
            ]);

            return new ClaimStatusChangesFetchResult(count($statusChanges), $e->getMessage(), false);
        }

        return new ClaimStatusChangesFetchResult(count($statusChanges), null, true);
    }

    private function insertClaimStatusChanges(array $statusChanges, Context $context): void
    {
        $upsertPayload = [];
        $fetchedClaims = [];

        /** @var ClaimStatusChange $statusChange */
        foreach ($statusChanges as $statusChange) {
            $fileNumber = $statusChange->getFileNumber();
            $claim      = null;

            if (!array_key_exists($fileNumber, $fetchedClaims)) {
                $claim                      = $this->claimService->getClaim($fileNumber, $context);
                $fetchedClaims[$fileNumber] = $claim;
            } else {
                $claim = $fetchedClaims[$fileNumber];
            }

            if (!$claim) {
                $this->logger->warning(sprintf('Claim status history: Could not find claim by file number %s', $fileNumber));

                continue;
            }

            $upsertPayload[] = [
                'fileNumber'               => $fileNumber,
                'invoiceNumber'            => $statusChange->getInvoiceNumber(),
                'claimId'                  => $claim->getId(),
                'changedAt'                => $statusChange->getTime(),
                'statusCode'               => $statusChange->getStatusCode(),
                'statusName'               => $statusChange->getStatusName(),
                'closingReasonCode'        => $statusChange->getClosingReasonCode(),
                'closingReasonExplanation' => $statusChange->getClosingReasonExplanation(),
                'closingReasonNote'        => $statusChange->getClosingReasonNote(),
            ];
        }

        if (empty($upsertPayload)) {
            return;
        }

        $this->claimStatusHistoryRepository->upsert($upsertPayload, $context);
    }
}
