<?php

declare(strict_types=1);

namespace Mefi\Inkasso\Components\Claim\Task;

use Exception;
use Psr\Log\LoggerInterface;
use Shopware\Core\Framework\Context;
use Shopware\Core\Framework\DataAbstractionLayer\EntityRepositoryInterface;
use Shopware\Core\Framework\MessageQueue\ScheduledTask\ScheduledTaskHandler;
use Mefi\Inkasso\Components\ConfigReader\ConfigReaderInterface;
use Mefi\Inkasso\Components\ConfigReader\Struct\Configuration;
use Mefi\Inkasso\Components\Claim\ClaimStatusHistoryServiceInterface;

class ClaimStatusHistorySyncHandler extends ScheduledTaskHandler
{
    /** @var ClaimStatusHistoryServiceInterface */
    private $claimStatusHistoryService;

    /** @var ConfigReaderInterface */
    private $configReader;

    /** @var LoggerInterface */
    private $logger;

    public function __construct(
        EntityRepositoryInterface $scheduledTaskRepository,
        ClaimStatusHistoryServiceInterface $claimStatusHistoryService,
        ConfigReaderInterface $configReader,
        LoggerInterface $logger
    ) {
        parent::__construct(
            $scheduledTaskRepository
        );

        $this->claimStatusHistoryService = $claimStatusHistoryService;
        $this->configReader              = $configReader;
        $this->logger                    = $logger;
    }

    public static function getHandledMessages(): iterable
    {
        return [
            ClaimStatusHistorySyncTask::class,
        ];
    }

    public function run(): void
    {
        $this->logger->info('Starting to synchronize all claim status changes from the mediafinanz API');

        $context = Context::createDefaultContext();

        $config = $this->configReader->read();
        // plugin configuration
        $clientId = $config->get(Configuration::CONFIG_CLIENT_ID);
        $clientLicence = $config->get(Configuration::CONFIG_CLIENT_LICENCE);

  /*      error_log(date('Y-m-d H:i:s') . "\tclientId: ". var_export($clientId,true) . "\n", 3, '/app/var/log/cron.log');
        error_log(date('Y-m-d H:i:s') . "\tclientLicence: ". var_export($clientLicence,true) . "\n", 3, '/app/var/log/cron.log');*/

        if ( empty($clientId) || empty($clientLicence) )
        {
            $this->logger->error('mediafinanz plugin configuration not complete', [
                'clientId'      => $clientId,
                'clientLicence' => $clientLicence,
            ]);

            return ;
        }

        try {
            $result = $this->claimStatusHistoryService->fetchClaimStatusChanges($context);

            $this->logger->info('Finished synchronization of claim status changes', [
                'total'        => $result->getTotal(),
                'success'      => $result->getSuccess(),
                'errorMessage' => $result->getErrorMessage(),
            ]);
        } catch (Exception $ex) {
            $this->logger->error('Synchronization of the claim status changes failed!', [
                'message' => $ex->getMessage(),
                'trace'   => $ex->getTraceAsString(),
            ]);
        }
    }
}
