<?php

declare(strict_types=1);

namespace Mefi\Inkasso\Components\Partner;

use Shopware\Core\Framework\Validation\DataBag\RequestDataBag;
use Mefi\Inkasso\Components\Partner\Struct\ClientRegistrationResult;
use Mefi\Inkasso\Components\Partner\Struct\RegistrationKeyGenerationResult;
use Mefi\Inkasso\Components\SoapAPI\MediafinanzSoapClientInterface;
use Mefi\Inkasso\Components\SoapAPI\Partner\Request\Builder\ClientRegistrationRequestBuilderInterface;
use Mefi\Inkasso\Components\SoapAPI\Partner\Request\GenerateRegistrationKeyRequest;
use Mefi\Inkasso\Components\SoapAPI\Partner\Response\RegisterClientResponse;
use Mefi\Inkasso\Components\SoapAPI\Partner\Validation\RegistrationValidatorInterface;

class PartnerRegistrationService implements PartnerRegistrationServiceInterface
{
    /** @var MediafinanzSoapClientInterface */
    private $partnerClient;

    /** @var ClientRegistrationRequestBuilderInterface */
    private $clientRegistrationRequestBuilder;

    /** @var RegistrationValidatorInterface */
    private $registrationValidator;

    public function __construct(
        MediafinanzSoapClientInterface $partnerClient,
        RegistrationValidatorInterface $registrationValidator,
        ClientRegistrationRequestBuilderInterface $clientRegistrationRequestBuilder
    ) {
        $this->partnerClient                    = $partnerClient;
        $this->clientRegistrationRequestBuilder = $clientRegistrationRequestBuilder;
        $this->registrationValidator            = $registrationValidator;
    }

    public function registerClient(RequestDataBag $registrationRequestData): ClientRegistrationResult
    {
        $validationResult = $this->registrationValidator->validate($registrationRequestData);

        if (!empty($validationResult)) {
            return new ClientRegistrationResult(false, null, $validationResult);
        }

        $clientRegistrationRequest = $this->clientRegistrationRequestBuilder->getData($registrationRequestData);

        $response = new RegisterClientResponse($this->partnerClient->sendRequest($clientRegistrationRequest));

        if (!$response->hasErrors()) {
            return new ClientRegistrationResult(true, $response->getClientId());
        }

        $errorMessage = $response->getHumanReadableErrorList();

        return new ClientRegistrationResult(false, null, $errorMessage);
    }

    public function generateRegistrationKey(): RegistrationKeyGenerationResult
    {
        $generateRegistrationKeyRequest = new GenerateRegistrationKeyRequest();

        $response = $this->partnerClient->sendRequest($generateRegistrationKeyRequest);

        if (is_string($response)) {
            return new RegistrationKeyGenerationResult(true, $response);
        }

        return new RegistrationKeyGenerationResult(false, null, 'Could not generate registration key');
    }
}
