<?php

declare(strict_types=1);

namespace Mefi\Inkasso\Components\SoapAPI;

use SoapClient;
use SoapFault;
use Psr\Log\LoggerInterface;
use Mefi\Inkasso\Components\ConfigReader\ConfigReaderInterface;
use Mefi\Inkasso\Components\ConfigReader\Struct\Configuration;
use Mefi\Inkasso\Components\SoapAPI\DataTypes\Auth;

abstract class MediafinanzSoapClient implements MediafinanzSoapClientInterface
{
    /** @var Auth */
    protected $authorization;

    /** @var string */
    protected $baseUrl = 'https://soap.mediafinanz.de';

    /**
     * @var null|SoapClient
     */
    protected $soapClient;

    /** @var ConfigReaderInterface */
    protected $configReader;

    /** @var LoggerInterface */
    private $logger;

    public function __construct(
        ConfigReaderInterface $configReader,
        LoggerInterface $logger
    ) {
        $this->configReader = $configReader;
        $this->logger       = $logger;
    }


    public function sendRequest(RequestInterface $request, string $salesChannelId = null)
    {
        if ($this->soapClient === null) {
            $this->soapClient = $this->getSoapClient();
        }

        if ($this->authorization === null) {
            $this->authorization = $this->getAuthorizationStruct($salesChannelId);
        }

        $functionName = $request->getFunctionName();
        $requestData  = $request->normalizeRequestData();
        $auth         = $this->authorization->jsonSerialize();

        $this->logger->debug(sprintf('Sending request to %s::%s', $this->getWsdlPath(), $functionName), [
            'payload' => $requestData,
        ]);

        try {

            $response = $this->soapClient->$functionName($auth, ...$requestData);

        } catch (SoapFault $soapFault) { /** @phpstan-ignore-line */

            $this->logger->error(__METHOD__ . '[[' . __LINE__ . ']] Exception:' . $soapFault->getMessage(), [
                'functionName' => $functionName,
                'SoapFault' => $soapFault,
            ]);

            $response = (object) [
                'errorList' =>  $soapFault->getMessage()
            ];
        }

        $this->logger->debug(sprintf('Received response from %s::%s', $this->getWsdlPath(), $functionName), [
            'payload' => $response,
        ]);

        return $response;
    }

    /**
     * @param Auth $auth
     */
    public function setAuthorization(Auth $auth): void
    {
        $this->authorization = $auth;
    }

    /**
     * Provide a configured soap client instance
     * @param array $config
     * @return SoapClient|null
     */
    protected function getSoapClient(array $config = self::DEFAULT_SOAP_CLIENT_CONFIG): ?SoapClient /** @phpstan-ignore-line */
    {
        $wsdlUrl = sprintf('%s/%s', $this->baseUrl, $this->getWsdlPath());

        try {

           $soapClient = new SoapClient($wsdlUrl, $config);

        } catch (SoapFault $soapFault) {

            $this->logger->error(__METHOD__ . '[[' . __LINE__ . ']] Exception:' . $soapFault->getMessage(), [
                'SoapFault' => $soapFault,
            ]);

            $soapClient = NULL;
        }

        return $soapClient;
    }

    protected function getAuthorizationStruct(string $salesChannelId = null): Auth
    {
        $config             = $this->configReader->read($salesChannelId);
        $clientId           = $config->get(Configuration::CONFIG_CLIENT_ID, 0);
        $clientLicence      = $config->get(Configuration::CONFIG_CLIENT_LICENCE);
        $sandboxMode        = $config->get(Configuration::CONFIG_SANDBOX_ACTIVE, false);
        $applicationLicence = $config->get(Configuration::CONFIG_APPLICATION_LICENCE);

        $auth = new Auth();
        $auth->setClientId($clientId);
        $auth->setSandbox($sandboxMode);
        $auth->setClientLicence($clientLicence);
        $auth->setApplicationLicence($applicationLicence);
        $auth->generateLicenceKey();

        return $auth;
    }
}
