<?php

declare(strict_types=1);

namespace Mefi\Inkasso\Components\SoapAPI\Partner\Request\Builder;

use Shopware\Core\Framework\Context;
use Shopware\Core\Framework\DataAbstractionLayer\EntityRepositoryInterface;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Criteria;
use Shopware\Core\Framework\Uuid\Uuid;
use Shopware\Core\Framework\Validation\DataBag\RequestDataBag;
use Shopware\Core\System\Country\CountryEntity;
use Shopware\Core\System\Salutation\SalutationEntity;
use Mefi\Inkasso\Components\SoapAPI\Partner\DataTypes\BankAccount;
use Mefi\Inkasso\Components\SoapAPI\Partner\DataTypes\ClientRegistration;
use Mefi\Inkasso\Components\SoapAPI\Partner\DataTypes\InternalParams;
use Mefi\Inkasso\Components\SoapAPI\Partner\Request\ClientRegistrationRequest;
use Mefi\Inkasso\Components\SoapAPI\Partner\Request\Struct\ClientRegistrationRequestStruct;

class ClientRegistrationRequestBuilder implements ClientRegistrationRequestBuilderInterface
{
    public const COUNTRY_DEFAULT = 'DE';

    /** @var EntityRepositoryInterface */
    private $countryRepository;

    /** @var EntityRepositoryInterface */
    private $salutationRepository;

    public function __construct(
        EntityRepositoryInterface $countryRepository,
        EntityRepositoryInterface $salutationRepository
    ) {
        $this->countryRepository    = $countryRepository;
        $this->salutationRepository = $salutationRepository;
    }

    public function getData(RequestDataBag $request): ClientRegistrationRequest
    {
        $requestData = new ClientRegistrationRequestStruct();
        $requestData->setClientRegistration($this->getClientRegistrationData($request));
        $requestData->setBankAccount(new BankAccount());
        $requestData->setInternalParams(new InternalParams());

        return new ClientRegistrationRequest($requestData);
    }

    private function getClientRegistrationData(RequestDataBag $request): ClientRegistration
    {
        $clientRegistration = new ClientRegistration();
        $clientRegistration->setPartnerCustomerId(Uuid::randomHex());
        $clientRegistration->setCompany($request->get('company', ''));
        $clientRegistration->setAddress($this->getSalutationCodeById($request->get('salutation', '')));
        $clientRegistration->setFirstname($request->get('firstname', ''));
        $clientRegistration->setLastname($request->get('lastname', ''));
        $clientRegistration->setLegalForm($request->get('legalForm', ''));
        $clientRegistration->setStreet($request->get('address', ''));
        $clientRegistration->setPostcode($request->get('zip', ''));
        $clientRegistration->setCity($request->get('city', ''));
        $clientRegistration->setCountry($this->getCountryCodeById($request->get('country', '')));
        $clientRegistration->setTelephone($request->get('phone', ''));
        $clientRegistration->setEmail($request->get('mail', ''));
        $clientRegistration->setFax($request->get('fax', ''));
        $clientRegistration->setHomepage($request->get('website', ''));
        $clientRegistration->setEstimatedClaims($request->get('estimatedClaims', null));

        return $clientRegistration;
    }

    private function getCountryCodeById(string $countryId): string
    {
        /** @var null|CountryEntity $countrySearchResult */
        $countrySearchResult = $this->countryRepository->search(new Criteria([$countryId]), Context::createDefaultContext())->first();

        if ($countrySearchResult === null) {
            return self::COUNTRY_DEFAULT;
        }

        return $countrySearchResult->getIso() ?? self::COUNTRY_DEFAULT;
    }

    private function getSalutationCodeById(string $salutationId): string
    {
        /** @var null|SalutationEntity $salutationSearchResult */
        $salutationSearchResult = $this->salutationRepository->search(new Criteria([$salutationId]), Context::createDefaultContext())->first();

        if ($salutationSearchResult === null) {
            return '';
        }

        return $salutationSearchResult->getSalutationKey();
    }
}
