<?php

declare(strict_types=1);

namespace Mefi\Inkasso\Components\Solvency\EventListeners;

use Psr\Log\LoggerInterface;
use Shopware\Core\Checkout\Customer\CustomerEntity;
use Shopware\Storefront\Page\Checkout\Finish\CheckoutFinishPage;
use Shopware\Storefront\Page\Checkout\Finish\CheckoutFinishPageLoadedEvent;
use Shopware\Core\Checkout\Order\OrderEntity;
use Shopware\Core\Checkout\Order\Aggregate\OrderTransaction\OrderTransactionStateHandler;
use Shopware\Core\Framework\Context;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Mefi\Inkasso\Components\ConfigReader\ConfigReaderInterface;
use Mefi\Inkasso\Components\ConfigReader\Struct\Configuration;
use Mefi\Inkasso\Components\Solvency\OrderSolvencyServiceInterface;
use Mefi\Inkasso\Components\Solvency\Struct\CustomerInfoStruct;
use Mefi\Inkasso\Components\Solvency\Validation\ScoreValidatorInterface;
use Mefi\Inkasso\Components\Solvency\Validation\Violation\AbstractScoreViolation;
use Mefi\Inkasso\Utils\AddressHelper;
use Mefi\Inkasso\Utils\SalesChannelHelper;

class SolvencyScoreEventListener implements EventSubscriberInterface
{
    public const ALLOWED_COUNTRIES = ['DE'];

    /** @var OrderSolvencyServiceInterface */
    private $orderSolvencyService;

    /** @var ScoreValidatorInterface */
    private $scoreValidator;

    /** @var ConfigReaderInterface */
    private $configReader;

    /** @var LoggerInterface */
    private $logger;

    /** @var CheckoutFinishPage */
    private $checkoutFinishPage;

    /** @var CustomerEntity */
    private $customer;

    /** @var OrderTransactionStateHandler */
    private $transactionStateHandler;

    public function __construct(
        OrderSolvencyServiceInterface $solvencyService,
        ScoreValidatorInterface $scoreValidator,
        ConfigReaderInterface $configReader,
        LoggerInterface $logger,
        OrderTransactionStateHandler $transactionStateHandler
    ) {
        $this->orderSolvencyService = $solvencyService;
        $this->scoreValidator       = $scoreValidator;
        $this->configReader         = $configReader;
        $this->logger               = $logger;
        $this->transactionStateHandler = $transactionStateHandler;
    }

    public static function getSubscribedEvents(): array
    {
        return [
            CheckoutFinishPageLoadedEvent::class => 'onCheckoutFinishPageLoaded',
        ];
    }


    public function onCheckoutFinishPageLoaded(CheckoutFinishPageLoadedEvent $pageLoadedEvent): void
    {

        $this->checkoutFinishPage = $pageLoadedEvent->getPage();
        $salesChannelContext    = $pageLoadedEvent->getSalesChannelContext();

       /* if ($salesChannelContext === null) {
            $this->logger->error('Solvency check: invalid request');
            return;
        }*/

        $this->customer = SalesChannelHelper::getCustomerForSolvencyChecks($salesChannelContext);

        if ($this->customer === null) {
            $this->logger->error('Solvency check: invalid customer', [
                'salesChannelContext' => $salesChannelContext
            ]);

            return;
        }

        if (!$this->shouldCheckSolvency()) {

            $this->logger->debug('Solvency check: solvency check not necessary', [
                'orderNumber' => $this->checkoutFinishPage->getOrder()->getOrderNumber()
            ]);
            return;
        }

        $config = $this->configReader->read($salesChannelContext->getSalesChannel()->getId());
        $this->checkSolvency($this->checkoutFinishPage->getOrder(), $salesChannelContext->getContext());

        if (!$config->get(Configuration::CONFIG_DIFFERENT_SHIPPING)) {
            $this->logger->debug('Solvency check: check shipping address disabled according to config', [
                'orderNumber' => $this->checkoutFinishPage->getOrder()->getOrderNumber()
            ]);

            return;
        }

       //Recheck solvency for the shipping address
        $shippingAddress = AddressHelper::getShippingAddressFromOrder($this->checkoutFinishPage->getOrder());
        $billingAddress  =  $this->checkoutFinishPage->getOrder()->getBillingAddress();

        if (!$shippingAddress || AddressHelper::equals($shippingAddress, $billingAddress)) {
            $this->logger->debug('Solvency check: shipping and billing address identical', [
                'shippingAddress' => $shippingAddress,
                'billingAddress' => $billingAddress
            ]);

            return;
        }

        $this->checkSolvency($this->checkoutFinishPage->getOrder(), $salesChannelContext->getContext(), false);
    }


    private function shouldCheckSolvency() : bool
    {

        $billingAddress = SalesChannelHelper::getBillingAddressForSolvencyChecks($this->customer);

        if ($billingAddress->getCountry() === null
            || !in_array($billingAddress->getCountry()->getIso(), self::ALLOWED_COUNTRIES, true)) {

            $this->logger->debug('Solvency check: country not allowed', [
                'billingAddress' => $billingAddress
            ]);

            return false;
        }

        $order = $this->checkoutFinishPage->getOrder();

        $transactions        = $order->getTransactions();
        $config              = $this->configReader->read($order->getSalesChannelId());
        $checkPaymentMethods = $config->get(Configuration::CONFIG_CRITICAL_PAYMENT_METHODS);

        if (empty($checkPaymentMethods)) {
            return false;
        }

        // plugin configuration
        $clientId = $config->get(Configuration::CONFIG_CLIENT_ID);
        $clientLicence = $config->get(Configuration::CONFIG_CLIENT_LICENCE);

        if ( empty($clientId) || empty($clientLicence) )
        {
            $this->logger->error('mediafinanz plugin configuration not complete', [
                'clientId'      => $clientId,
                'clientLicence' => $clientLicence,
            ]);
            return false;
        }

        $checkAmount   = (float) $config->get(Configuration::CONFIG_INFORMATION_MIN_VALUE);
        $checkSolvency = false;

        //Order amount does not match the requirement at all
        if ($checkAmount > 0 && $order->getAmountTotal() < $checkAmount) {
            return $checkSolvency;
        }


        //Order without payment
        $activeTransaction = ( $transactions !== null ) ? $transactions->last() : null;
        if ($activeTransaction === null) {
            return $checkSolvency;
        }

//        $transactionId = $transactions->last()->getId();

        if (in_array($activeTransaction->getPaymentMethodId(), $checkPaymentMethods, true)) {
            $checkSolvency = true;
        }

        return $checkSolvency;
    }

    private function checkSolvency(OrderEntity $orderEntity, Context $context, bool $useBillingAddress = true): void
    {
        $address = $useBillingAddress
            ? $orderEntity->getBillingAddress()
            : AddressHelper::getShippingAddressFromOrder($orderEntity);

        if (!$address) {

            $this->logger->error('Solvency check: invalid address', [
                'order' => $orderEntity
            ]);

            return;
        }

        $customerInformation = new CustomerInfoStruct($address, $this->customer);

        $score = $this->orderSolvencyService->getScoreFromOrder($customerInformation, $orderEntity, $context);

        $violations = $this->scoreValidator->validate(
            $score,
            $customerInformation,
            $orderEntity->getAmountTotal(),
            $orderEntity->getSalesChannelId(),
            $context
        );

        if ($violations->count() > 0) {
            $violation  = $violations->get(0);
            $apiMessage = $violation->getMessage();

            if ($violation instanceof  AbstractScoreViolation) {
                $apiMessage = $violation->getOriginalMessage();
            }

            $this->logger->debug(sprintf('Solvency check for order %s rejected', $orderEntity->getOrderNumber()), [
                'orderId'           => $orderEntity->getId(),
                'useBillingAddress' => $useBillingAddress,
                'scoreErrors'       => $score->getErrors() !== null ? $score->getErrors()->jsonSerialize() : [],
                'violation'         => $apiMessage,
                'product'           => $score->getProduct(),
            ]);

            // throw new SolvencyScoreException($violations, [])
            $this->checkoutFinishPage->setPaymentFailed(true);

             $transactions        = $orderEntity->getTransactions();
             $transactionId = $transactions->last()->getId();

             $this->transactionStateHandler->fail($transactionId, $context);

             return;
        }

        $this->logger->debug(sprintf('Solvency check for order %s was successful', $orderEntity->getOrderNumber()), [
           'orderId'           => $orderEntity->getId(),
           'useBillingAddress' => $useBillingAddress,
           'product'           => $score->getProduct(),
        ]);
    }
}
