<?php

declare(strict_types=1);

namespace Mefi\Inkasso\Components\Solvency;

use DateTimeImmutable;
use Shopware\Core\Checkout\Cart\Exception\CustomerNotLoggedInException;
use Shopware\Core\Checkout\Order\Aggregate\OrderAddress\OrderAddressEntity;
use Shopware\Core\Framework\Context;
use Shopware\Core\Framework\DataAbstractionLayer\EntityRepositoryInterface;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Criteria;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Filter\EqualsFilter;
use Mefi\Inkasso\Components\ConfigReader\ConfigReaderInterface;
use Mefi\Inkasso\Components\ConfigReader\Struct\Configuration;
use Mefi\Inkasso\Core\DataAbstractionLayer\SolvencyCheckHistory\SolvencyCheckHistoryEntity;
use Mefi\Inkasso\Utils\AddressHelper;
use Mefi\Inkasso\Components\SoapAPI\Buergel\DataTypes\BuergelSuspect;

class SolvencyCachingService implements SolvencyCachingServiceInterface
{
    /** @var EntityRepositoryInterface */
    private $historyRepository;

    /** @var ConfigReaderInterface */
    private $configReader;

    public function __construct(EntityRepositoryInterface $historyRepository, ConfigReaderInterface $configReader)
    {
        $this->historyRepository = $historyRepository;
        $this->configReader      = $configReader;
    }

    public function getScoreById(string $scoreCacheEntityId, Context $context): ?SolvencyCheckHistoryEntity
    {
        $historyCriteria = new Criteria([$scoreCacheEntityId]);
        $historyCriteria->addAssociation('customer');

        return $this->historyRepository->search($historyCriteria, $context)->first();
    }

    public function getScoreFromCache(string $customerId, string $product, OrderAddressEntity $address, Context $context): ?SolvencyCheckHistoryEntity
    {
        if (empty($customerId)) {
            throw new CustomerNotLoggedInException();
        }

        $addressHash = AddressHelper::getHashFromAddress($address);

        return $this->getCustomerSolvencyFromCache($customerId, $product, $addressHash, $context);
    }

    public function isCacheScoreValid(?SolvencyCheckHistoryEntity $historyResult, string $salesChannelId = null): bool
    {
        if ($historyResult === null) {
            return false;
        }

        $invalidateCacheDays = $this->configReader->read($salesChannelId)->get(Configuration::CONFIG_TIMING_CACHE, 30);

        if ($historyResult->getCreatedAt() !== null
        && (int) ($historyResult->getCreatedAt()->diff(new DateTimeImmutable()))->format('%a') <= $invalidateCacheDays) {
            return true;
        }

        if ($historyResult->getUpdatedAt() !== null
        && (int) ($historyResult->getUpdatedAt()->diff(new DateTimeImmutable()))->format('%a') <= $invalidateCacheDays) {
            return true;
        }

        return false;
    }

    public function writeScoreToCache(
        ?string $id,
        string $customerId,
        string $customerNumber,
        BuergelSuspect $suspect,
        string $addressHash,
        int $justification,
        float $totalAmount,
        string $product,
        float $score,
        ?string $responseText,
        Context $context
    ): string {

        $result = $this->historyRepository->upsert([[
            'id'             => $id,
            'customerId'     => $customerId,
            'customernumber'  => $customerNumber,
            'company' => $suspect->getCompany(),
            'firstname' => $suspect->getFirstname(),
            'lastname' => $suspect->getLastname(),
            'street' => $suspect->getStreet() . " ". $suspect->getHouseNumber(),
            'postcode' => $suspect->getPostcode(),
            'city' => $suspect->getCity(),
            'addressHash'    => $addressHash,
            'amount'         => $totalAmount,
            'product'        => $product,
            'score'          => $score,
            'responseText'   => $responseText,
            'justification'  => $justification,
        ]], $context);

        return (string) current($result->getPrimaryKeys('mefi_inkasso_solvency_check_history'));
    }

    public function updateScore(string $scoreId, ?float $newScore, ?string $responseText, Context $context): void
    {
        $this->historyRepository->upsert([[
            'id'           => $scoreId,
            'score'        => $newScore,
            'responseText' => $responseText,
            'updatedAt'    => new DateTimeImmutable(),
        ]], $context);
    }

    private function getCustomerSolvencyFromCache(string $customerId, string $product, string $addressHash, Context $context): ?SolvencyCheckHistoryEntity
    {
        $historyCriteria = new Criteria();
        $historyCriteria->addFilter(new EqualsFilter('addressHash', $addressHash));
        //$historyCriteria->addFilter(new EqualsFilter('product', $product));

        return $this->historyRepository->search($historyCriteria, $context)->first();
    }
}
