<?php

declare(strict_types=1);

namespace Mefi\Inkasso\Components\Solvency;

use Psr\Log\LoggerInterface;
use Shopware\Core\Framework\Context;
use Mefi\Inkasso\Components\ConfigReader\ConfigReaderInterface;
use Mefi\Inkasso\Components\ConfigReader\Struct\Configuration;
use Mefi\Inkasso\Components\Solvency\Struct\CustomerInfoStruct;
use Mefi\Inkasso\Components\Solvency\Struct\ScoreResult;
use Mefi\Inkasso\Utils\AddressHelper;

class SolvencyScoreAdapter
{
    /** @var SolvencyScoreApiServiceInterface */
    private $solvencyApiService;

    /** @var SolvencyCachingServiceInterface */
    private $solvencyCachingService;

    /** @var LoggerInterface */
    private $logger;

    /** @var ConfigReaderInterface */
    private $configReader;

    public function __construct(
        SolvencyScoreApiServiceInterface $solvencyApiService,
        SolvencyCachingServiceInterface $solvencyCachingService,
        ConfigReaderInterface $configReader,
        LoggerInterface $logger
    ) {
        $this->solvencyApiService     = $solvencyApiService;
        $this->solvencyCachingService = $solvencyCachingService;
        $this->logger                 = $logger;
        $this->configReader           = $configReader;
    }

    public function fetchScore(
        CustomerInfoStruct $customerInformation,
        string $mediafinanzProduct,
        float $totalAmount,
        string $salesChannelId,
        int $justification,
        Context $context,
        bool $enableCaching = true
    ): ScoreResult {
        $cacheId       = null;
        $pluginConfig  = $this->configReader->read($salesChannelId);
        $responseError = null;

        //First try to load from cache
        if ($enableCaching && $this->shouldUseCache($pluginConfig, $totalAmount)) {

            $historyEntry = $this->solvencyCachingService->getScoreFromCache($customerInformation->getCustomerId(), $mediafinanzProduct, $customerInformation->getAddress(), $context);
            $isValid      = $historyEntry ? $this->solvencyCachingService->isCacheScoreValid($historyEntry, $salesChannelId) : false;

            if ($isValid) {
                $this->logger->debug(sprintf('Solvency adapter: Loaded %s from cache', $historyEntry->getId()));

                return new ScoreResult($historyEntry->getScore(), null, $historyEntry->getResponseText(), $mediafinanzProduct);
            }

            //Found a cache entry but it's not up to date!
            if ($historyEntry) {
                $cacheId = $historyEntry->getId();
                $this->logger->debug(sprintf('Solvency adapter: Found a cache entry %s but it is not up to date', $cacheId ));
            }
        }

        //Load from API
        $apiScore = $this->solvencyApiService->getScore(
            $customerInformation->getAddress(),
            $customerInformation->getEmail(),
            $customerInformation->isB2bCustomer(),
            $salesChannelId,
            $justification,
            $mediafinanzProduct
        );

        if ($apiScore->getErrors() !== null && $apiScore->getErrors()->count() > 0) {
            $responseError = $apiScore->getErrors()->__toString();
            $this->logger->error(sprintf('Solvency adapter: Could not fetch solvency score from API due to the following errors: %s', $apiScore->getErrors()));
        } else {
            $suspect = $this->solvencyApiService->getRequestData()->getVars()['suspect'];

            $addressHash = AddressHelper::getHashFromAddress($customerInformation->getAddress());

            //Write back to cache
            $result = $this->solvencyCachingService->writeScoreToCache(
                $cacheId,
                $customerInformation->getCustomerId(),
                $customerInformation->getCustomerNumber(),
                $suspect,
                $addressHash,
                $justification,
                $totalAmount,
                $mediafinanzProduct,
                $apiScore->getScore() ? $apiScore->getScore() : 0,
                $apiScore->getMessage(),
                $context
            );

            if (!empty($result) )
            {
                $apiScore->setHistoryId($result);
            } else {
                $this->logger->error('Solvency adapter: Could not write score to DB', $suspect->getVars() );
            }
        }

        return $apiScore;
    }

    /**
     * Checks if the cache should actually be used for the given amount.
     */
    private function shouldUseCache(Configuration $pluginConfig, float $totalAmount): bool
    {
        $invalidCacheMinValue = $pluginConfig->get(Configuration::CONFIG_TIMING_NEW_MIN_VALUE, null);
        $invalidCacheMaxValue = $pluginConfig->get(Configuration::CONFIG_TIMING_NEW_MAX_VALUE, null);

        // cache exclusions
        if ($invalidCacheMinValue !== null && $totalAmount <= $invalidCacheMinValue) {
            $this->logger->error('Solvency adapter: MinValue has impact on cache exclusion', [
                'invalidCacheMinValue' => $invalidCacheMinValue,
                'invalidCacheMaxValue' => $invalidCacheMaxValue,
                'totalAmount'          => $totalAmount
            ]);

            return false;
        }

        if ($invalidCacheMaxValue !== null && $totalAmount >= $invalidCacheMaxValue) {
            $this->logger->error('Solvency adapter: MaxValue has  impact on cache exclusion', [
                'invalidCacheMinValue' => $invalidCacheMinValue,
                'invalidCacheMaxValue' => $invalidCacheMaxValue,
                'totalAmount'          => $totalAmount
            ]);

            return false;

        }

        return true;
    }
}
