<?php

declare(strict_types=1);

namespace Mefi\Inkasso\Components\Solvency;

use Shopware\Core\Checkout\Order\Aggregate\OrderAddress\OrderAddressEntity;
use Shopware\Core\Framework\Context;
use Mefi\Inkasso\Components\ConfigReader\ConfigReader;
use Mefi\Inkasso\Components\ConfigReader\Struct\Configuration;
use Mefi\Inkasso\Components\SoapAPI\Buergel\Request\Builder\ConCheckRequestBuilderInterface;
use Mefi\Inkasso\Components\SoapAPI\Buergel\Request\Builder\RiskCheckRequestBuilderInterface;
use Mefi\Inkasso\Components\SoapAPI\Buergel\Response\ConCheckScoreResponse;
use Mefi\Inkasso\Components\SoapAPI\Buergel\Response\RiskCheckScoreResponse;
use Mefi\Inkasso\Components\SoapAPI\MediafinanzSoapClientInterface;
use Mefi\Inkasso\Components\Solvency\Product\Factory\SolvencyProductFactoryInterface;
use Mefi\Inkasso\Components\Solvency\Product\SolvencyProductInterface;
use Mefi\Inkasso\Components\Solvency\Struct\ScoreResult;
use Mefi\Inkasso\Components\SoapAPI\Buergel\Request\Struct\ConCheckScoreRequestStruct;
use Shopware\Core\Framework\Struct\Struct;

class SolvencyScoreApiService implements SolvencyScoreApiServiceInterface
{
    /**
     * The shop admin can select "none" in the product selection inside the plugin config.
     * In this case no API action should be triggered and the solvency score should be processed as "success".
     */
    public const PRODUCT_NONE = 'none';

    /** @var SolvencyProductFactoryInterface */
    private $productFactory;

    /** @var ConfigReader */
    private $configReader;

    /** @var ConCheckRequestBuilderInterface */
    private $conCheckRequestBuilder;

    /** @var MediafinanzSoapClientInterface */
    private $client;

    /** @var RiskCheckRequestBuilderInterface */
    private $riskCheckRequestBuilder;

    /** @var ReferenceProviderInterface */
    private $referenceProvider;

    /** @var  Struct */
    protected $requestData;

    public function __construct(
        SolvencyProductFactoryInterface $productFactory,
        ConfigReader $configReader,
        ConCheckRequestBuilderInterface $conCheckRequestBuilder,
        MediafinanzSoapClientInterface $client,
        RiskCheckRequestBuilderInterface $riskCheckRequestBuilder,
        ReferenceProviderInterface $referenceProvider
    ) {
        $this->productFactory          = $productFactory;
        $this->configReader            = $configReader;
        $this->conCheckRequestBuilder  = $conCheckRequestBuilder;
        $this->client                  = $client;
        $this->riskCheckRequestBuilder = $riskCheckRequestBuilder;
        $this->referenceProvider       = $referenceProvider;
    }

    public function getScore(
        OrderAddressEntity $address,
        string $customerEmail,
        bool $isB2bCustomer,
        string $salesChannelId,
        int $justification,
        string $selectedProduct = null
    ): ScoreResult {
        $pluginConfig = $this->configReader->read($salesChannelId);

        if (empty($selectedProduct)) {
            $selectedProduct = $isB2bCustomer ? $pluginConfig->get(Configuration::CONFIG_B2B_PRODUCT, null) : $pluginConfig->get(Configuration::CONFIG_B2C_PRODUCT, null);
        }

        // Skip API calls if no product was selected.
        if ($selectedProduct === self::PRODUCT_NONE) {
            return new ScoreResult(0, null, 'No product selected', $selectedProduct);
        }

        $productService = $this->productFactory->getProduct($selectedProduct);

        $result = $isB2bCustomer
            ? $this->getRiskScore($productService, $address, $customerEmail, $salesChannelId, $justification)
            : $this->getConCheckScore($productService, $address, $customerEmail, $salesChannelId, $justification);

        $result->setProduct($selectedProduct);

        return $result;
    }

    private function getRiskScore(
        SolvencyProductInterface $productService,
        OrderAddressEntity $address,
        string $customerEmail,
        string $salesChannelId,
        int $justification = ConCheckScoreRequestStruct::JUSTIFICATION_SOLVENCY_CHECK
    ): ScoreResult {
        $request     = $productService->getRequest();
        $this->requestData = $this->riskCheckRequestBuilder->getData($address, $customerEmail, $justification, $this->getReference());

        $request->setRequestData($this->requestData);

        $response = new RiskCheckScoreResponse(
            $this->client->sendRequest($request, $salesChannelId)
        );

        if ($response->hasErrors() || $response->getScoreResult() === null) {
            return new ScoreResult(null, $response->getErrors());
        }

        return new ScoreResult(
            $response->getScoreResult()->getScore(),
            $response->getErrors(),
            $response->getScoreResult()->getScoreText()
        );
    }

    private function getConCheckScore(
        SolvencyProductInterface $productService,
        OrderAddressEntity $address,
        string $customerEmail,
        string $salesChannelId,
        int $justification = ConCheckScoreRequestStruct::JUSTIFICATION_SOLVENCY_CHECK
    ): ScoreResult {
        $request     = $productService->getRequest();
        $this->requestData = $this->conCheckRequestBuilder->getData($address, $customerEmail, $justification, $this->getReference() );

        $request->setRequestData($this->requestData);

        $response = new ConCheckScoreResponse(
            $this->client->sendRequest($request, $salesChannelId)
        );

        if ($response->hasErrors() || $response->getScoreResult() === null) {
            return new ScoreResult(null, $response->getErrors());
        }

        return new ScoreResult($response->getScoreResult()->getScore(), $response->getErrors(), $response->getScoreResult()->getScoreText());
    }

    private function getReference(): ?string
    {
        $context = Context::createDefaultContext();

        return $this->referenceProvider->getReference($context);
    }

    public function getRequestData() : Struct
    {
        return $this->requestData;
    }
}
