<?php

declare(strict_types=1);

namespace Mefi\Inkasso\Components\Solvency\Validation;

use Shopware\Core\Framework\Context;
use Symfony\Component\Validator\ConstraintViolationList;
use Mefi\Inkasso\Components\ConfigReader\ConfigReaderInterface;
use Mefi\Inkasso\Components\ConfigReader\Struct\Configuration;
use Mefi\Inkasso\Components\SoapAPI\DataTypes\Error;
use Mefi\Inkasso\Components\Solvency\Rule\ScoringRuleProviderInterface;
use Mefi\Inkasso\Components\Solvency\SolvencyScoreApiService;
use Mefi\Inkasso\Components\Solvency\Struct\CustomerInfoStruct;
use Mefi\Inkasso\Components\Solvency\Struct\ScoreResult;
use Mefi\Inkasso\Components\Solvency\Validation\Violation\ScoreErrorViolation;
use Mefi\Inkasso\Components\Solvency\Validation\Violation\ScoreViolation;
use Mefi\Inkasso\Components\Solvency\Validation\Violation\UnknownPersonViolation;

class ScoreValidator implements ScoreValidatorInterface
{
    public const VIOLATION_SCORE_UNKNOWN_OR_TOO_LOW = 'SOLVENCY_CHECK_FAILED';

    public const VIOLATION_PERSON_UNKNOWN = 'SOLVENCY_CHECK_FAILED_UNKNOWN_PERSON';

    public const VIOLATION_API_ERROR_GENERIC = 'SOLVENCY_CHECK_FAILED_DUE_TO_ERROR';

    private const DEFAULT_API_ERROR_SCORE_TOO_LOW = 'Der Score dieser Person reicht für den angefragten Betrag nicht aus.<br/>Die Person hat einen Score von %s, mindestens erforderlich: %s';

    /**
     * If this is part of a score message treat person as "Unknown"
     */
    private const MATCH_ERROR_STRING_UNKNOWN_PERSON = 'Person unbekannt';

    /** @var ScoringRuleProviderInterface */
    private $scoringProvider;

    /** @var ConfigReaderInterface */
    private $configReader;

    public function __construct(ScoringRuleProviderInterface $scoringProvider, ConfigReaderInterface $configReader)
    {
        $this->scoringProvider = $scoringProvider;
        $this->configReader    = $configReader;
    }

    public function validate(ScoreResult $score, CustomerInfoStruct $customerInfoStruct, float $totalAmount, string $salesChannelId, Context $context): ConstraintViolationList
    {
        $violationList = new ConstraintViolationList();

        $rule = $customerInfoStruct->isB2bCustomer() ? $this->scoringProvider->getB2bRule($totalAmount, $salesChannelId)
                                                     : $this->scoringProvider->getB2cRule($totalAmount, $salesChannelId);

        $config = $this->configReader->read($salesChannelId);

        $declineUnknownPerson = !$customerInfoStruct->isB2bCustomer() ? (bool) $config->get(Configuration::CONFIG_DECLINE_UNKNOWN_PERSON)
                                                                      : false;

        $declineUnknownScore = !$customerInfoStruct->isB2bCustomer() ? (bool) $config->get(Configuration::CONFIG_DECLINE_UNKNOWN_SCORE_B2C)
                                                                     : (bool) $config->get(Configuration::CONFIG_DECLINE_UNKNOWN_SCORE_B2B);

        $errorNotificationMessage = $this->getDisplayMessage($salesChannelId, $context->getLanguageId());

        //Score is too low
        if ($score->getScore() && $rule->getScore() < $score->getScore()) {
            $violationList->add($this->buildScoreViolation(sprintf(self::DEFAULT_API_ERROR_SCORE_TOO_LOW, $score->getScore(), $rule->getScore()), $errorNotificationMessage));
        }

        //Regular score errors like invalid address
        if ($score->getErrors() !== null) {
            foreach ($score->getErrors() as $error) {
                $violationList->add($this->buildScoreErrorViolation($error));
            }
        }

        //Customer is unknown
        if ($declineUnknownPerson && !empty($score->getMessage()) && stripos($score->getMessage(), self::MATCH_ERROR_STRING_UNKNOWN_PERSON) !== false) {
            $violationList->add($this->buildUnknownPersonViolation($score->getMessage() ?: '', $errorNotificationMessage));
        }

        //Score is unknown
        if ($declineUnknownScore
            && $score->getProduct() !== SolvencyScoreApiService::PRODUCT_NONE
            && ($score->getScore() === null || $score->getScore() < 0.1)
        ) {
            $violationList->add($this->buildScoreViolation($score->getMessage() ?: '', $errorNotificationMessage));
        }

        return $violationList;
    }

    private function buildScoreViolation(string $apiMessage, string $displayMessage): ScoreViolation
    {
        $violation = new ScoreViolation(
            Configuration::DEFAULT_SOLVENCY_CHECK_ERROR_NOTICE,
            Configuration::DEFAULT_SOLVENCY_CHECK_ERROR_NOTICE,
            [],
            '/',
            $displayMessage,
            'score',
            null,
            self::VIOLATION_SCORE_UNKNOWN_OR_TOO_LOW
        );

        $violation->setOriginalMessage($apiMessage);

        return $violation;
    }

    private function buildScoreErrorViolation(Error $error): ScoreErrorViolation
    {
        $violation = new ScoreErrorViolation(
            $error->getMessage(),
            $error->getRaw(),
            ['code' => $error->getCode()],
            '/',
            $error->getMessage(),
            $error->getCode(),
            null,
            self::VIOLATION_API_ERROR_GENERIC);

        $violation->setOriginalMessage($error->getMessage());

        return $violation;
    }

    private function buildUnknownPersonViolation(string $apiMessage, string $displayMessage): UnknownPersonViolation
    {
        $violation = new UnknownPersonViolation(
            Configuration::DEFAULT_SOLVENCY_CHECK_ERROR_NOTICE,
            Configuration::DEFAULT_SOLVENCY_CHECK_ERROR_NOTICE,
            [],
            '/',
            $displayMessage,
            'unknownPerson',
            null,
            self::VIOLATION_PERSON_UNKNOWN
        );

        $violation->setOriginalMessage($apiMessage);

        return $violation;
    }

    private function getDisplayMessage(?string $salesChannelId = null, ?string $languageId = null): string
    {
        $config        = $this->configReader->read($salesChannelId);
        $notifications = $config->get(Configuration::CONFIG_NOTIFICATION);

        if ((!empty($notifications) && $languageId !== null) && array_key_exists($languageId, $notifications) && !empty($notifications[$languageId])) {
            return $notifications[$languageId];
        }

        return Configuration::DEFAULT_SOLVENCY_CHECK_ERROR_NOTICE;
    }
}
