<?php

declare(strict_types=1);

namespace Mefi\Inkasso\Core\EventListener;

use Psr\Log\LoggerInterface;
use Shopware\Core\Framework\Validation\BuildValidationEvent;
use Shopware\Core\PlatformRequest;
use Shopware\Core\System\SalesChannel\SalesChannelContext;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpFoundation\RequestStack;
use Mefi\Inkasso\Components\ConfigReader\ConfigReaderInterface;
use Mefi\Inkasso\Components\ConfigReader\Struct\Configuration;
use Mefi\Inkasso\Components\Solvency\Validation\Constraint\SolvencyCheckbox;
use Mefi\Inkasso\Utils\SalesChannelHelper;

class BuildValidationEventListener implements EventSubscriberInterface
{
    public const FIELD_SOLVENCY = 'mediafinanz_solvency';

    /** @var RequestStack */
    private $requestStack;

    /** @var ConfigReaderInterface */
    private $configReader;

    /** @var LoggerInterface */
    private $logger;

    public function __construct(RequestStack $requestStack, ConfigReaderInterface $configReader, LoggerInterface $logger)
    {
        $this->requestStack = $requestStack;
        $this->configReader = $configReader;
        $this->logger       = $logger;

    }

    public static function getSubscribedEvents(): array
    {
        return [
            'framework.validation.order.create' => 'onBuildValidation',
            'framework.validation.order.update' => 'onBuildValidation',
        ];
    }

    public function onBuildValidation(BuildValidationEvent $event): void
    {
        $currentRequest = $this->requestStack->getCurrentRequest();

        if ($currentRequest === null) {
            return;
        }

        /** @var null|SalesChannelContext $salesChannelContext */
        $salesChannelContext = $currentRequest->get(PlatformRequest::ATTRIBUTE_SALES_CHANNEL_CONTEXT_OBJECT);

        if ($salesChannelContext === null) {
            return;
        }

        if (!SalesChannelHelper::isCountryAllowedForSolvencyChecks($salesChannelContext, $this->logger)) {
            return;
        }

        $pluginConfig           = $this->configReader->read($salesChannelContext->getSalesChannel()->getId());
        $criticalPaymentMethods = $pluginConfig->get(Configuration::CONFIG_CRITICAL_PAYMENT_METHODS, []);
        $paymentMethod          = $salesChannelContext->getPaymentMethod();

        if (in_array($paymentMethod->getId(), $criticalPaymentMethods, true)) {
            $event->getDefinition()->add(self::FIELD_SOLVENCY, new SolvencyCheckbox());
        }
    }
}
