import template from './mefi-claim-list-content.html.twig';

const Criteria = Shopware.Data.Criteria;
const { Component, Mixin } = Shopware;

Component.register('mefi-claim-list-content', {
    template,

    inject: [
        'repositoryFactory',
        'MefiInkassoClaimService'
    ],

    mixins: [
        Mixin.getByName('listing'),
        Mixin.getByName('notification')
    ],

    data() {
        return {
            claims: null,
            limit: 25,
            sortBy: 'claimDetail.timeOfTransmission',
            bookDirectPayment: {},
            showBookDirectPaymentModal: false,
            sortDirection: 'DESC',
            isLoading: false,
            showCloseModal: false,
            filterLoading: false
        };
    },

    props: {
        customerId: {
            type: String,
            required: false
        }
    },

    metaInfo() {
        return {
            title: this.$createTitle()
        };
    },

    computed: {
        claimRepository() {
            return this.repositoryFactory.create('mefi_inkasso_claim');
        },

        claimColumns() {
            return [{
                property: 'fileNumber',
                label: 'mefi-claim.list.fileNumber',
                routerLink: 'mefi.claim.detail'
            }, {
                property: 'order.orderNumber',
                label: 'mefi-claim.list.orderNumber'
            }, {
                property: 'firstName',
                label: 'mefi-claim.list.customerName'
            }, {
                property: 'invoiceNumber',
                label: 'mefi-claim.list.invoiceNumber'
            }, {
                property: 'amount',
                label: 'mefi-claim.list.claimAmount'
            }, {
                property: 'statusHistory',
                label: 'mefi-claim.list.status'
            }, {
                property: 'claimDetail.timeOfTransmission',
                label: 'mefi-claim.list.claimDate'
            }];
        }
    },

    methods: {
        getList() {
            this.isLoading = true;

            const claimListCriteria = new Criteria(this.page, this.limit);
            claimListCriteria.addAssociation('claimCustomer');
            claimListCriteria.addAssociation('claimDetail');
            claimListCriteria.addAssociation('order');
            claimListCriteria.addAssociation('customer');
            claimListCriteria.addAssociation('statusHistory');

            if (this.customerId) {
                claimListCriteria.addFilter(Criteria.equals('customerId', this.customerId));
            }

            return this.claimRepository.search(claimListCriteria, Shopware.Context.api)
                .then((result) => {
                    this.claims = result;
                    this.total = result.total;
                    this.page = result.criteria.page;
                    this.limit = result.criteria.limit;
                }).finally(() => {
                    this.isLoading = false;
                });
        },

        onShowCloseModal(claimEntityId) {
            this.showCloseModal = claimEntityId;
        },

        onShowBookDirectPaymentModal(claimEntityId) {
            this.showBookDirectPaymentModal = claimEntityId;
        },

        onCloseModal() {
            this.showCloseModal = false;
            this.showBookDirectPaymentModal = false;
        },

        onCloseClaim(claimEntity) {
            this.onCloseModal();

            // Wait for the modal to close and then continue to close the claim.
            this.$nextTick().then(() => {
                this.isLoading = true;

                this.MefiInkassoClaimService.closeClaim(claimEntity.fileNumber).then((response) => {
                    if (response.success) {
                        this.createNotificationSuccess({
                            title: this.$tc('mefi-claim.closeClaim.successTitle'),
                            message: this.$tc('mefi-claim.closeClaim.successMessage')
                        });
                    } else {
                        this.createNotificationError({
                            title: this.$tc('global.default.error'),
                            message: response.errorMessage
                        });
                    }
                }).catch((error) => {
                    this.showErrorResponseNotification(error);
                }).finally(() => {
                    this.isLoading = false;
                });
            });
        },

        onBookDirectPayment(claimEntity) {
            if (!this.bookDirectPayment.paidAmount || !this.bookDirectPayment.dateOfPayment) {
                this.createNotificationError({
                    title: this.$tc('global.default.error'),
                    message: this.$tc('mefi-claim.bookDirectPayment.invalidFormError')
                });

                return;
            }

            this.onCloseModal();

            // Wait for the modal to close and then continue to close the claim.
            this.$nextTick().then(() => {
                this.isLoading = true;

                this.MefiInkassoClaimService.bookDirectPayment(this.bookDirectPayment, claimEntity).then((response) => {
                    if (response.success) {
                        this.createNotificationSuccess({
                            title: this.$tc('mefi-claim.bookDirectPayment.successTitle'),
                            message: this.$tc('mefi-claim.bookDirectPayment.successMessage')
                        });
                    } else {
                        this.createNotificationError({
                            title: this.$tc('global.default.error'),
                            message: this.$tc('mefi-claim.bookDirectPayment.errorMessage')
                        });
                    }
                }).catch((error) => {
                    this.showErrorResponseNotification(error);
                }).finally(() => {
                    this.isLoading = false;
                });
            });
        },

        showErrorResponseNotification(error) {
            const errorResponse = error.response;
            let errorMessage = this.$tc('mefi-claim.closeClaim.errorMessage');

            if (errorResponse.data.errors[0] !== null) {
                errorMessage = errorResponse.data.errors[0].detail;
            }

            this.createNotificationError({
                title: this.$tc('global.default.error'),
                message: errorMessage
            });
        }
    }
});
