import template from './mefi-claim-detail.html.twig';

const Criteria = Shopware.Data.Criteria;
const { Component, Mixin } = Shopware;

Component.register('mefi-claim-detail', {
    template,

    mixins: [
        Mixin.getByName('notification')
    ],

    inject: [
        'repositoryFactory',
        'MefiInkassoClaimService',
        'systemConfigApiService'
    ],

    data() {
        return {
            claimId: null,
            claim: {},
            claimDetail: {
                type: '1' // Defaults to 'Goods sold'
            },
            claimCustomer: {},
            isLoading: false,
            isNewClaim: false,
            selectedOrder: {},
            selectedOrderId: null,
            selectedCustomerId: null,
            selectedCustomer: {},
            existingClaim: false,
            configDomain: 'MefiInkasso.settings',
            isCustomerReferrer: false,
            datepickerConfig: {
                enableTime: false,
                dateFormat: 'd.m.Y',
                altFormat: 'd.m.Y'
            }
        };
    },

    metaInfo() {
        return {
            title: this.$createTitle(this.identifier)
        };
    },

    computed: {
        getClaimStatus() {
            if (!this.claim) {
                return;
            }

            if (this.claim.statusHistory && this.claim.statusHistory.last()) {
                const status = this.claim.statusHistory.last();

                return status.statusName.concat(' [', status.statusCode, ']');
            }

            return this.$tc('mefi-claim.list.statusEmpty');
        },

        claimRepository() {
            return this.repositoryFactory.create('mefi_inkasso_claim');
        },

        customerRepository() {
            return this.repositoryFactory.create('customer');
        },

        documentRepository() {
            return this.repositoryFactory.create('document');
        },

        orderCriteria() {
            const criteria = new Criteria();

            if (this.selectedCustomerId) {
                criteria.addFilter(Criteria.equals('order.orderCustomer.customerId', this.selectedCustomerId));
            }

            return criteria;
        },

        customerCriteria() {
            const criteria = new Criteria();
            criteria.addAssociation('defaultBillingAddress');
            criteria.addAssociation('defaultBillingAddress.country');

            return criteria;
        },

        claimTypes() {
            return [
                { value: '1', label: this.$tc('mefi-claim.details.claimType.goods') },
                { value: '2', label: this.$tc('mefi-claim.details.claimType.prepaid') },
                { value: '3', label: this.$tc('mefi-claim.details.claimType.service') }
            ];
        }
    },

    created() {
        this.claimId = this.$route.params.id;

        this.setCustomerIfProvided();
        this.getEntityData();
    },

    methods: {
        getEntityData() {
            if (this.claimId === 'new') {
                this.isNewClaim = true;

                return;
            }

            this.isLoading = true;

            const claimCriteria = new Criteria();
            claimCriteria.addAssociation('claimCustomer');
            claimCriteria.addAssociation('claimDetail');
            claimCriteria.addAssociation('order');
            claimCriteria.addAssociation('customer');
            claimCriteria.addAssociation('statusHistory');
            claimCriteria.addAssociation('customer.defaultBillingAddress');

            this.claimRepository
                .get(this.claimId, Shopware.Context.api, claimCriteria)
                .then((entity) => {
                    this.isLoading = false;

                    this.claim = entity;
                    this.claimDetail = entity.claimDetail;
                    this.claimCustomer = entity.claimCustomer;

                    this.selectedCustomerId = entity.customerId;
                    this.selectedCustomer = entity.customer;
                    this.selectedOrderId = entity.orderId;
                    this.selectedOrder = entity.order;
                }).catch((error) => {
                    this.createNotificationError({
                        title: this.$tc('mefi-claim.details.loadEntityError'),
                        message: error
                    });
                });
        },

        transformToFormData() {
            const formData = this.claim;

            if (this.selectedOrderId) {
                formData.orderId = this.selectedOrderId;
            }

            if (this.selectedCustomerId) {
                formData.customerId = this.selectedCustomerId;
            }

            formData.claimDetail = this.claimDetail;
            formData.claimCustomer = this.claimCustomer;

            return formData;
        },

        onChangeSelectedCustomer(customerId, customerEntity) {
            if (customerId === null) {
                return;
            }

            this.claimCustomer = customerEntity;
            this.selectedCustomer = customerEntity;
            this.selectedCustomerId = customerId;

            if (customerEntity.defaultBillingAddress) {
                this.claimCustomer.street = customerEntity.defaultBillingAddress.street;
                this.claimCustomer.city = customerEntity.defaultBillingAddress.city;
                this.claimCustomer.countryId = customerEntity.defaultBillingAddress.country.id;
                this.claimCustomer.zipCode = customerEntity.defaultBillingAddress.zipcode;
            }
        },

        onChangeSelectedOrder(orderId, orderEntity) {
            if (orderId === null) {
                return;
            }

            this.checkIfClaimExistsForOrder(orderId);

            this.claim.order = orderEntity;
            this.selectedOrderId = orderId;
            this.selectedOrder = orderEntity;

            this.claimDetail.amount = orderEntity.amountTotal;

            const customerNumber = orderEntity.orderCustomer.customerNumber;
            const orderNumber = orderEntity.orderNumber;

            this.claim.invoiceNumber = `K${customerNumber}#B${orderNumber}`;
            this.setInvoiceNumber(orderId);

            this.readConfigDefaults();
        },

        readConfigDefaults() {
            this.isLoading = true;

            this.systemConfigApiService.getValues(this.configDomain)
                .then((config) => {
                    const dateOfLastReminder = new Date(this.selectedOrder.orderDate);

                    let dateOfLastReminderDays = config['MefiInkasso.settings.firstDunning'];
                    let dunningCost = config['MefiInkasso.settings.baseDunningCosts'];
                    let type = config['MefiInkasso.settings.defaultTypeReminder'];

                    if (!dunningCost) {
                        dunningCost = 0;
                    }

                    if (!type) {
                        type = '1';
                    }

                    if (!dateOfLastReminderDays) {
                        dateOfLastReminderDays = 30;
                    }

                    dateOfLastReminder.setDate(dateOfLastReminder.getDate() + dateOfLastReminderDays);

                    this.claimDetail.cost = dunningCost;
                    this.claimDetail.type = type;
                    this.claimDetail.dateOfLastReminder = dateOfLastReminder;
                }).finally(() => {
                    this.isLoading = false;
                });
        },

        setInvoiceNumber(orderId) {
            const invoiceCriteria = new Criteria();
            invoiceCriteria.addAssociation('documentType');
            invoiceCriteria.addFilter(Criteria.equals('orderId', orderId));
            invoiceCriteria.addFilter(Criteria.equals('documentType.technicalName', 'invoice'));

            this.documentRepository
                .search(invoiceCriteria, Shopware.Context.api)
                .then((result) => {
                    if (this.isValidInvoiceResult(result)) {
                        this.claim.invoiceNumber += '#R' + result.first().config.documentNumber;
                    }
                }).catch((error) => {
                    this.createNotificationError({
                        title: this.$tc('mefi-claim.details.loadEntityError'),
                        message: error
                    });
                });
        },

        isValidInvoiceResult(result) {
            return (
                result.first() &&
                result.first().config &&
                result.first().config.documentNumber
            );
        },

        resetFormData() {
            this.selectedCustomerId = null;
            this.selectedOrderId = null;
            this.claim = {};
            this.claimDetail = {
                type: '1' // Defaults to 'Goods sold'
            };
            this.claimCustomer = {};
            this.selectedOrder = {};
            this.existingClaim = false;
            this.setCustomerIfProvided();
        },

        onSubmitClaim() {
            this.isLoading = true;
            const formData = this.transformToFormData();

            this.MefiInkassoClaimService.newClaim(formData)
                .then((response) => {
                    if (response.success) {
                        this.resetFormData();

                        this.createNotificationSuccess({
                            title: this.$tc('mefi-claim.newClaim.successfulClaimTitle'),
                            message: this.$tc('mefi-claim.newClaim.successfulClaimMessage')
                        });
                    } else {
                        this.createNotificationError({
                            title: this.$tc('mefi-claim.newClaim.errorDetailTitle'),
                            message: response.errorMessage
                        });
                    }
                }).catch((error) => {
                    this.showErrorResponseNotification(error);
                }).finally(() => {
                    this.isLoading = false;
                });
        },

        showErrorResponseNotification(error) {
            const errorResponse = error.response;
            let errorMessage = this.$tc('mefi-claim.newClaim.soapErrorReferLog');

            if (errorResponse.data.errors[0] !== null) {
                errorMessage = errorResponse.data.errors[0].detail;
            }

            this.createNotificationError({
                title: this.$tc('global.default.error'),
                message: errorMessage
            });
        },

        checkIfClaimExistsForOrder(orderId) {
            const claimCriteria = new Criteria();
            claimCriteria.addFilter(Criteria.equals('orderId', orderId));

            this.claimRepository.search(claimCriteria, Shopware.Context.api).then((searchResult) => {
                if (searchResult.first()) {
                    this.existingClaim = searchResult.first();
                } else {
                    this.existingClaim = false;
                }
            });
        },

        getFullName(item) {
            return `${item.firstName} ${item.lastName}`;
        },

        isFormInvalid() {
            // Validate selected order & customer
            if (!this.selectedOrder || !this.selectedCustomer || !this.selectedOrder.orderDate) {
                return true;
            }

            // Claim base validation
            if (!this.claim.invoiceNumber) {
                return true;
            }

            // Claim specific customer info validation
            if (!this.claimCustomer.firstName
                || !this.claimCustomer.lastName
                || !this.claimCustomer.street
                || !this.claimCustomer.zipCode
                || !this.claimCustomer.city
                || !this.claimCustomer.countryId
            ) {
                return true;
            }

            // Claim detail specific information validation
            if (!this.claimDetail.reason
                || !this.claimDetail.type
                || !this.claimDetail.amount
                || isNaN(this.claimDetail.cost)
                || !this.claimDetail.dateOfLastReminder
            ) {
                return true;
            }

            // claim exists
            if (this.existingClaim)
            {
                return true;
            }

            return false;
        },

        setCustomerIfProvided() {
            const routeParams = this.$route.params;

            if (routeParams.customerId && routeParams.customerId.length > 0) {
                this.isLoading = true;

                this.customerRepository.get(routeParams.customerId, Shopware.Context.api, this.customerCriteria)
                    .then((customer) => {
                        this.isCustomerReferrer = true;
                        this.onChangeSelectedCustomer(routeParams.customerId, customer);
                    }).finally(() => {
                        this.isLoading = false;
                    });
            }
        }
    }
});
