import template from './mefi-configuration-index.html.twig';
import './mefi-configuration-index.scss';

const { Component, Mixin } = Shopware;

Component.register('mefi-configuration-index', {
    template,

    mixins: [
        Mixin.getByName('notification'),
        Mixin.getByName('sw-inline-snippet')
    ],

    inject: [
        'MefiInkassoConfigService',
        'systemConfigApiService'
    ],

    data() {
        return {
            selectedSalesChannelId: null,
            fallbackConfig: {},
            isLoading: false,
            config: {},
            configDomain: 'MefiInkasso.settings',
            isTestSuccessful: false,
            isTesting: false,
            isRegistrationRequired: true,
        };
    },

    created() {
        this.loadConfig();
        this.isRegistrationRequired = this.checkForRegistrationRequired();
    },

    computed: {
        swContext() {
            const context = { ...Shopware.Context.api, inheritance: true };

            (async () => {
                context.languageId = await Shopware
                    .Service('localeToLanguageService')
                    .localeToLanguage(this.$root.$i18n.locale);
            })();

            return context;
        }
    },

    metaInfo() {
        return {
            title: this.$createTitle()
        };
    },

    methods: {
        getDefaultRules() {
            return [
                {
                    id: this.generateInternalItemId(),
                    minAmount: 0.0,
                    maxAmount: 100.0,
                    score: 2.80
                },
                {
                    id: this.generateInternalItemId(),
                    minAmount: 100.0,
                    maxAmount: 500.0,
                    score: 2.80
                },
                {
                    id: this.generateInternalItemId(),
                    minAmount: 500.0,
                    maxAmount: 0.0,
                    score: 2.80
                }
            ];
        },

        /**
         * Loading configuration elements, remove domain and arrange them in a global config array
         */
        loadConfig() {
            const result = {};
            this.isTestSuccessful = false;
            this.isTesting = false;
            this.isLoading = true;

            this.systemConfigApiService.getValues(this.configDomain, this.selectedSalesChannelId)
                .then((config) => {
                    Object.keys(config).forEach((key) => {
                        const configActualKey = key.replace(`${this.configDomain}.`, '');

                        result[configActualKey] = config[key];
                    });

                    this.config = this.applyDefaultSettings(result);

                    if (this.selectedSalesChannelId === null) {
                        this.fallbackConfig = this.config;
                    }

                    this.isLoading = false;
                });
        },

        applyDefaultSettings(config) {
            if (!config.b2cScoringRules) {
                config.b2cScoringRules = this.getDefaultRules();
            }

            if (!config.b2bScoringRules) {
                config.b2bScoringRules = this.getDefaultRules();
            }

            if (!config.checkMinValue) {
                config.checkMinValue = 20;
            }

            if (!config.cacheInvalidationTime) {
                config.cacheInvalidationTime = 30;
            }


            if (!config.b2cProduct) {
                config.b2cProduct = 'buergel_basic';
            }

            if (!config.b2bProduct) {
                config.b2bProduct = 'risk_normal';
            }

            if (!config.firstDunning) {
                config.firstDunning = 14;
            }

            if (!config.defaultTypeReminder) {
                config.defaultTypeReminder = 1;
            }

            if (!config.baseDunningCosts && config.baseDunningCosts !== 0) {
                config.baseDunningCosts = 5.00;
            }

            return config;
        },

        /**
         * Trigger catches event of changing salesChannelId and force reloading configuration
         * @param salesChannelId
         */
        onChangeSalesChannel(salesChannelId) {
            if (!this.selectedSalesChannelId) {
                this.fallbackConfig = this.config;
            }

            if (!salesChannelId) {
                this.fallbackConfig = null;
            }

            this.selectedSalesChannelId = salesChannelId;

            this.loadConfig();
        },

        /**
         * Called if the global save button has been clicked. Saves the configuration.
         */
        onSave() {
            this.isLoading = true;

            if (!this.validateConfig()) {
                this.isLoading = false;
                return;
            }

            const configData = {};

            Object.keys(this.config).forEach((key) => {
                const configActualKey = `${this.configDomain}.${key}`;
                configData[configActualKey] = this.config[key];
            });

            this.systemConfigApiService.saveValues(configData, this.selectedSalesChannelId)
                .then(() => {
                    if (this.selectedSalesChannelId === null) {
                        this.fallbackConfig = configData;
                    }

                    this.createNotificationSuccess({
                        title: this.$tc('mefi-configuration.config.titleSaveSuccess'),
                        message: this.$tc('mefi-configuration.config.messageSaveSuccess')
                    });

                    this.isLoading = false;
                }).catch((error) => {
                    this.createNotificationError({
                        title: this.$tc('mefi-configuration.config.invalid.title'),
                        message: error
                    });
                });
        },

        /**
         * Generates a random string which is being used to identify the item inside the grid.
         *
         * @return {string}
         */
        generateInternalItemId() {
            return Math.random().toString(36).substring(2) + Date.now().toString(36);
        },

        validateConfig() {
            if (this.config.firstDunning === 0 || isNaN(this.config.firstDunning)) {
                this.config.firstDunning = 30;
            }

            return true;
        },

        checkForRegistrationRequired() {
            this.isLoading = true;
            this.MefiInkassoConfigService.isClientRegistered()
                .then((response) => {
                    if (response.hasOwnProperty('isClientRegistered')) {
                        this.isRegistrationRequired = !response.isClientRegistered;
                    }
                })
                .finally(() => {
                    this.isLoading = false;
                });
        }
    }
});
