import template from './mefi-solvency-create.html.twig';
import './mefi-solvency-create.scss';

const { Component, Mixin } = Shopware;
const Criteria = Shopware.Data.Criteria;

Component.register('mefi-solvency-create', {
    template,

    mixins: [
        Mixin.getByName('notification')
    ],

    inject: ['repositoryFactory', 'MefiInkassoSolvencyApiService'],

    computed: {
        historyRepository() {
            return this.repositoryFactory.create('mefi_inkasso_solvency_check_history');
        },

        customerAddressCriteria() {
            let criteria = new Criteria();
            if (this.solvencyCheckData && this.solvencyCheckData.customerId) {
                // nur Kunden mit Land='DE'
                criteria.addAssociation('country');
                criteria.addFilter(Criteria.equals('country.iso', 'DE'));
                criteria.addFilter(Criteria.equals('customerId', this.solvencyCheckData.customerId));
            }

            return criteria;
        },

        customerCriteria() {
            let criteria = new Criteria();
            // nur Adressen mit Land='DE'
            criteria.addAssociation('defaultBillingAddress');
            criteria.addAssociation('defaultBillingAddress.country');
            criteria.addFilter(Criteria.equals('defaultBillingAddress.country.iso', 'DE'));
            return criteria;
        },

        justificationReason() {
            return [
                { id: 1, name: this.$tc('mefi-configuration.justification.reason.credit_request') },
                { id: 2, name: this.$tc('mefi-configuration.justification.reason.business_initiation') },
                { id: 3, name: this.$tc('mefi-configuration.justification.reason.solvency_check') },
                { id: 4, name: this.$tc('mefi-configuration.justification.reason.claim') }
            ];
        },

        productDataSourceB2c() {
            return [
                { id: 'buergel_basic', name: this.$tc('mefi-configuration.products.buergel_basic') },
                { id: 'buergel_normal', name: this.$tc('mefi-configuration.products.buergel_normal') },
                { id: 'buergel_plus', name: this.$tc('mefi-configuration.products.buergel_plus') },
                { id: 'schufa', name: this.$tc('mefi-configuration.products.schufa'), disabled: true }
            ];
        },

        productDataSourceB2b() {
            return [
                { id: 'risk_normal', name: this.$tc('mefi-configuration.products.risk_normal') },
                { id: 'risk_advanced', name: this.$tc('mefi-configuration.products.risk_advanced') },
                { id: 'risk_professional', name: this.$tc('mefi-configuration.products.risk_professional') }
            ];
        }
    },

    data() {
        return {
            isLoading: false,
            historyEntry: null,
            solvencyJustification: [],
            solvencyCheckData: {},
            solvencyProducts: [],
            scoreResult: null,
            scoreError: null,
            requiredFormData: ['customerId', 'customerAddressId', 'product', 'justification', 'qualifiedInterest']
        };
    },

    created() {
        this.loadHistoryData();
    },

    methods: {
        loadHistoryData() {
            this.solvencyProducts = this.productDataSourceB2c;
            this.solvencyJustification = this.justificationReason;
        },

        onPerform() {
            if (!this.isFormValid()) {
                return;
            }

            this.scoreResult = null;
            this.scoreError = null;

            this.isLoading = true;

            this.MefiInkassoSolvencyApiService.performNewSolvencyCheck(this.solvencyCheckData)
                .then((response) => {
                    if (response.data) {
                        this.scoreResult = response.data.score;
                    }

                    if (response.success && response.data && response.data.score) {

                        this.isLoading = false;
                        this.$router.push({name: 'mefi.solvency.history.detail', params: { historyId: response.data.score.historyId, notify: true }});
                        return;
                    }

                    // API error
                    if (response.error.message) {
                        this.scoreError = response.error.apiMessage;

                        this.createNotificationError({
                            title: this.$tc('mefi-solvency-history.create.perform.response.error.title'),
                            message: this.$tc('mefi-solvency-history.create.perform.response.error.errorMessage', 0, {
                                message: this.scoreError
                            })
                        });
                    } else {
                        // Score validation error
                        this.scoreError = this.$tc(`mefi-solvency-history.create.solvencyInfo.violation.${response.error.code}`);

                        this.createNotificationError({
                            title: this.$tc('mefi-solvency-history.create.perform.response.error.title'),
                            message: this.$tc('mefi-solvency-history.create.perform.response.error.errorMessage', 0, {
                                message: this.scoreError
                            })
                        });
                    }
                }).finally(() => {
                    this.isLoading = false;
                });
        },

        onChangeCustomerAddress(newAddressId, data) {
            if (!data.company || data.company.length < 1) {
                this.solvencyProducts = this.productDataSourceB2c;
            } else {
                this.solvencyProducts = this.productDataSourceB2b;
            }

            if (!this.solvencyProducts.some(solvencyProduct => solvencyProduct.value === this.solvencyCheckData.product)) {
                this.solvencyCheckData.product = null;
            }
        },

        isFormValid() {
            let validationResult = true;
            this.requiredFormData.forEach((formData) => {
                if (validationResult && !this.solvencyCheckData[formData]) {
                    this.createNotificationError({
                        title: this.$tc('mefi-solvency-history.create.perform.missing.title'),
                        message: this.$tc(`mefi-solvency-history.create.perform.missing.${formData}`)
                    });

                    validationResult = false;
                }
            });

            return validationResult;
        }
    }
});
