<?php

declare(strict_types=1);

namespace Mefi\Inkasso\Storefront\EventListener;

use Psr\Log\LoggerInterface;
use Shopware\Core\Checkout\Payment\PaymentMethodCollection;
use Shopware\Storefront\Page\Account\Order\AccountEditOrderPageLoadedEvent;
use Shopware\Storefront\Page\Checkout\Confirm\CheckoutConfirmPageLoadedEvent;
use Shopware\Storefront\Page\PageLoadedEvent;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Mefi\Inkasso\Components\ConfigReader\ConfigReaderInterface;
use Mefi\Inkasso\Components\ConfigReader\Struct\Configuration;
use Mefi\Inkasso\Storefront\PageExtension\CheckoutPageExtension;
use Mefi\Inkasso\Utils\SalesChannelHelper;


class CheckoutEventListener implements EventSubscriberInterface
{
    /** @var ConfigReaderInterface */
    private $configReader;

    /** @var LoggerInterface */
    private $logger;

    public function __construct(ConfigReaderInterface $configReader,  LoggerInterface $logger)
    {
        $this->configReader = $configReader;
        $this->logger       = $logger;
    }

    public static function getSubscribedEvents(): array
    {
        return [
            CheckoutConfirmPageLoadedEvent::class  => 'onCheckoutLoaded',
            AccountEditOrderPageLoadedEvent::class => 'onPaymentEditLoaded',
        ];
    }


    public function onPaymentEditLoaded(PageLoadedEvent $pageLoadedEvent): void
    {

        if (!$pageLoadedEvent instanceof AccountEditOrderPageLoadedEvent) {
            return;
        }

        $page = $pageLoadedEvent->getPage();
        $salesChannelContext    = $pageLoadedEvent->getSalesChannelContext();
        $pluginConfig           = $this->configReader->read($salesChannelContext->getSalesChannel()->getId());
        $criticalPaymentMethods = $pluginConfig->get(Configuration::CONFIG_CRITICAL_PAYMENT_METHODS, []);

        if (empty($criticalPaymentMethods) ) {
        //    || !in_array($salesChannelContext->getPaymentMethod()->getId(), $criticalPaymentMethods, true)) {

            $this->logger->debug(__METHOD__ .  ': no critical payment method', [
                'salesChannelPaymentMethod' => $salesChannelContext->getPaymentMethod()->getName()
            ]);

            return;
        }

        $order = $page->getOrder();
        $transactions = $order->getTransactions();

        //Order without payment
        $activeTransaction = $transactions->last();
        if ($transactions === null || $activeTransaction === null) {

            $this->logger->error('CheckoutEvent: no transaction found', [
                'orderTransaction' => $transactions
            ]);
            return;
        }

        if (    $activeTransaction->getStateMachineState()->getTechnicalName() === 'failed' &&
                in_array($activeTransaction->getPaymentMethodId(),$criticalPaymentMethods, true)
        ) {
            $this->logger->debug('CheckoutEvent: hide critical payment methods', [
                'orderNumber' => $order->getOrderNumber()
            ]);

            $paymentMethods = $page->getPaymentMethods();
            $paymentList = [];
            foreach($paymentMethods AS $paymentMethod)
            {
                if (!in_array($paymentMethod->getId(), $criticalPaymentMethods, true)) {
                   $paymentList [] = $paymentMethod;
                }
            }

            $page->setPaymentMethods( new PaymentMethodCollection($paymentList) );
            return;
        }

        $this->onCheckoutLoaded($pageLoadedEvent);
    }

    public function onCheckoutLoaded(PageLoadedEvent $pageLoadedEvent): void
    {
        if (!$pageLoadedEvent instanceof CheckoutConfirmPageLoadedEvent
            && !$pageLoadedEvent instanceof AccountEditOrderPageLoadedEvent) {
            return;
        }

        if (SalesChannelHelper::isCountryAllowedForSolvencyChecks($pageLoadedEvent->getSalesChannelContext(), $this->logger) !== true) {
            return;
        }

        $salesChannelContext    = $pageLoadedEvent->getSalesChannelContext();
        $pluginConfig           = $this->configReader->read($salesChannelContext->getSalesChannel()->getId());
        $criticalPaymentMethods = $pluginConfig->get(Configuration::CONFIG_CRITICAL_PAYMENT_METHODS, []);

        $page = $pageLoadedEvent->getPage();
        $paymentMethods = $page->getPaymentMethods();

        $paymentMethodValid = false;
        foreach($paymentMethods AS $paymentMethod)
        {
            if ( $paymentMethod->getId()===$salesChannelContext->getPaymentMethod()->getId() )
            {
                $paymentMethodValid = true;
                break;
            }
        }

        if ($paymentMethodValid === false
            || empty($criticalPaymentMethods)
            || !in_array($salesChannelContext->getPaymentMethod()->getId(), $criticalPaymentMethods, true)
        ) {

            $this->logger->debug(__METHOD__ . ': no critical payment method', [
                'salesChannelPaymentMethod' => $salesChannelContext->getPaymentMethod()->getName()
            ]);

            return;
        }

        $page                  = $pageLoadedEvent->getPage();
        $checkoutPageExtension = new CheckoutPageExtension(true);

        $page->addExtension(CheckoutPageExtension::EXTENSION_NAME, $checkoutPageExtension);
    }
}
