<?php

declare(strict_types=1);

namespace Mefi\Inkasso\Utils\Connection;

use Psr\Log\LoggerInterface;
use Mefi\Inkasso\Components\ConfigReader\ConfigReaderInterface;
use Mefi\Inkasso\Components\ConfigReader\Struct\Configuration;
use Mefi\Inkasso\Components\SoapAPI\DataTypes\Auth;
use Mefi\Inkasso\Components\SoapAPI\Encashment\Request\ClientClaimListRequest;
use Mefi\Inkasso\Components\SoapAPI\MediafinanzSoapClientInterface;
use Mefi\Inkasso\Components\SoapAPI\Encashment\Response\ClientClaimListResponse;
use Mefi\Inkasso\Utils\Connection\Struct\ValidationResult;

class ConnectionValidator implements ConnectionValidatorInterface
{
    /** @var MediafinanzSoapClientInterface */
    private $client;

    /** @var ConfigReaderInterface */
    private $configReader;

    /** @var LoggerInterface */
    private $logger;

    public function __construct(
        MediafinanzSoapClientInterface $client,
        ConfigReaderInterface $configReader,
        LoggerInterface $logger
    ) {
        $this->client       = $client;
        $this->configReader = $configReader;
        $this->logger       = $logger;
    }

    /**
     * Tests connection to soap api with given auth data
     * by creating and performing an example test call
     */
    public function testConnection(int $clientId, string $clientLicence): ValidationResult
    {
        $testAuthorization = $this->getTestAuthorizationStruct(
            $clientId,
            $clientLicence
        );

//        $request = new ClaimAccountingChangesRequest(null);
        $request = new ClientClaimListRequest(null);

        try {
           $this->client->setAuthorization($testAuthorization);

           $response = new ClientClaimListResponse(
               (object) $this->client->sendRequest($request)
           );

        } catch (\Exception $exp) {
            $this->logger->error('Request failed!' . $exp->getMessage());

            return new ValidationResult(false, $exp->getMessage(),[]);
        }

        if ($response->hasErrors())
        {
            return new ValidationResult(false, $response->getErrors()->first()->getMessage(),[]);
        }

        return new ValidationResult(true, null, $response->getClaimList());
    }

    /**
     * @param int $clientId
     * @param string $clientLicence
     * @return Auth
     */
    private function getTestAuthorizationStruct(int $clientId, string $clientLicence): Auth
    {
        $config             = $this->configReader->read();
        $sandboxMode        = $config->get(Configuration::CONFIG_SANDBOX_ACTIVE, false);
        $applicationLicence = $config->get(Configuration::CONFIG_APPLICATION_LICENCE );

        $auth = new Auth();
        $auth->setClientId($clientId);
        $auth->setSandbox($sandboxMode);
        $auth->setClientLicence($clientLicence);
        $auth->setApplicationLicence($applicationLicence);
        $auth->generateLicenceKey();

        return $auth;
    }
}
